package io.github.thehrz.infernalmobs

import io.github.thehrz.infernalmobs.handler.EffectHandle
import io.github.thehrz.infernalmobs.koin.pluginModule
import io.github.thehrz.infernalmobs.services.ConfigService
import io.github.thehrz.infernalmobs.services.InfernalMobService
import io.github.thehrz.infernalmobs.utils.ItemUtils
import org.bukkit.Bukkit
import org.bukkit.Material
import org.bukkit.NamespacedKey
import org.bukkit.enchantments.Enchantment
import org.bukkit.event.Listener
import org.bukkit.inventory.ItemStack
import org.bukkit.inventory.ShapedRecipe
import org.bukkit.plugin.java.JavaPlugin
import org.koin.core.KoinApplication
import org.koin.dsl.koinApplication
import org.koin.ksp.generated.defaultModule
import java.io.File
import java.util.logging.Level

class InfernalMobs : JavaPlugin() {
    var serverTime: Long = 0L

    lateinit var koinApp: KoinApplication

    companion object {
        lateinit var instance: InfernalMobs
    }

    override fun onEnable() {
        koinApp = koinApplication {
            modules(
                pluginModule(this@InfernalMobs),
                defaultModule
            )
        }

        // Registering event
        logger.info("Registering event listeners...")
        val listeners = koinApp.koin.getAll<Listener>()
        logger.info("Total possible listeners: ${listeners.size}")
        listeners.forEach {
            logger.info("Registering event listener: ${it::class.simpleName}")
            Bukkit.getPluginManager().registerEvents(it, this)
        }

        logger.info("Registered Events.")


        //Folder
        val dir = File(dataFolder.parentFile.path, this.name)
        if (!dir.exists()) dir.mkdir()


        //Old config check
//        if (File(dataFolder, "config.yml").exists()) {
//            if (config.getString("configVersion") == null) {
//                logger.log(Level.INFO, "No config version found!")
//                config["configVersion"] = Bukkit.getVersion()
//                    .split(":".toRegex())
//                    .dropLastWhile { it.isEmpty() }
//                    .toTypedArray()[1]
//                    .replace(")", "")
//                    .trim { it <= ' ' }
//                saveConfig()
//            }
//            if (Bukkit.getVersion() != config.getString("configVersion")) {
//                println(Bukkit.getVersion() + " contains " + config.getString("configVersion"))
//                logger.log(Level.INFO, "Old config found, deleting!")
//                File(dataFolder.toString() + File.separator + "config.yml").delete()
//            }
//        }

        //Register Config

//        if (!File(dataFolder, "config.yml").exists()) {
//            logger.log(Level.INFO, "No config.yml found, generating...")
//            saveConfig()
//            reloadConfig()
//        }


//        //Register Loots
//        if (!lootYML.exists()) {
//            logger.log(Level.INFO, "No loot.yml found, generating...")
//            reloadLoot()
//        }
//
//        //Register Save File
//        if (!saveYML.exists()) {
//            try {
//                saveYML.createNewFile()
//            } catch (e: IOException) {
//                e.printStackTrace()
//            }
//        }
        koinApp.koin.get<ConfigService>().loadConfigs()

        koinApp.koin.get<EffectHandle>().applyEffect()

        koinApp.koin.get<InfernalMobService>().reloadPowers()

        koinApp.koin.get<EffectHandle>().showEffect()

        addRecipes()
    }

//    fun reloadLoot() {
//        if (this.lootYML == null) {
//            this.lootYML = File(dataFolder, "loot.yml")
//        }
//        this.configService.lootConfig = YamlConfiguration.loadConfiguration(this.lootYML)
//
//        val defConfig = YamlConfiguration.loadConfiguration(lootYML)
//        configService.lootConfig.setDefaults(defConfig)
//    }

    val diviningStaff: ItemStack
        get() {
            val s = ItemUtils.getItem(
                Material.BLAZE_ROD,
                "§6§lDivining Rod",
                1,
                mutableListOf("Click to find infernal mobs.")
            )
            val m = s.itemMeta
            m.addEnchant(Enchantment.CHANNELING, 1, true)
            s.setItemMeta(m)
            return s
        }

    private fun addRecipes() {
        val staff = diviningStaff
        val key = NamespacedKey(this, "divining_staff")
        val sr = ShapedRecipe(key, staff)
        sr.shape("ANA", "ASA", "ASA")
        sr.setIngredient('N', Material.NETHER_STAR)
        sr.setIngredient('S', Material.BLAZE_ROD)
        Bukkit.addRecipe(sr)
    }
}