package io.github.thehrz.infernalmobs.commands

import io.github.thehrz.infernalmobs.InfernalMobs
import io.github.thehrz.infernalmobs.enums.Abilitiy
import io.github.thehrz.infernalmobs.gui.GUI
import io.github.thehrz.infernalmobs.handler.InfernalMobHandler
import io.github.thehrz.infernalmobs.handler.LootHandler
import io.github.thehrz.infernalmobs.models.InfernalMob
import io.github.thehrz.infernalmobs.services.ConfigService
import io.github.thehrz.infernalmobs.services.InfernalMobService
import io.github.thehrz.infernalmobs.utils.*
import org.bukkit.Bukkit
import org.bukkit.Location
import org.bukkit.Material
import org.bukkit.World
import org.bukkit.command.Command
import org.bukkit.command.CommandExecutor
import org.bukkit.command.CommandSender
import org.bukkit.command.TabCompleter
import org.bukkit.entity.*
import org.bukkit.inventory.ItemStack
import org.koin.core.annotation.Single
import java.util.*
import java.util.function.Consumer
import java.util.logging.Level
import java.util.stream.Collectors

@Single
class Command(
    val plugin: InfernalMobs,
    val configService: ConfigService,
    val infernalMobService: InfernalMobService,
    val lootHandler: LootHandler,
    val infernalMobHandler: InfernalMobHandler,
    val gui: GUI
) : CommandExecutor, TabCompleter {
    var errorList = mutableListOf<Player>()
    private val loops = 0

    init {
        plugin.getCommand("im")?.let { cmd ->
            cmd.setExecutor(this)
            cmd.tabCompleter = this
        }
    }

    private val randomMob: String
        get() {
            val mobList = configService.mainConfig.enabledMobs
            if (mobList.isEmpty()) {
                return "Zombie"
            }
            val mob = mobList[MathUtils.rand(1, mobList.size) - 1]
            return mob
        }

    override fun onCommand(sender: CommandSender, cmd: Command, label: String, args: Array<String>?): Boolean {
        if ((cmd.name.equals("infernalmobs", ignoreCase = true)) || (cmd.name.equals("im", ignoreCase = true))) {
            try {
                if (sender !is Player) {
                    if (!args.isNullOrEmpty() && (!args[0].equals(
                            "cspawn",
                            ignoreCase = true
                        )) && (!args[0].equals("pspawn", ignoreCase = true)) && (!args[0].equals(
                            "giveloot",
                            ignoreCase = true
                        )) && (!args[0].equals("reload", ignoreCase = true)) && (!args[0].equals(
                            "killall",
                            ignoreCase = true
                        ))
                    ) {
                        sender.sendMessage("This command can only be run by a player!")
                        return true
                    }
                }

                val player = sender as Player
                if (sender.hasPermission("infernal_mobs.commands")) {
                    if (args!!.isEmpty()) {
                        throwError(sender)
                        return true
                    }
                    if (args[0].equals("slotTest", ignoreCase = true)) {
                        for (i in configService.mainConfig.enabledCharmSlots) {
                            player.inventory.setItem(
                                i, ItemStack(
                                    Material.RED_STAINED_GLASS_PANE
                                )
                            )
                        }

                    } else if ((args.size == 1) && (args[0].equals("fixloot", ignoreCase = true))) {
                        //TODO: ? what is items
//                        val list =
//                            configService.mainConfig.items.getKeys(false)
//
//                        for (i in configService.lootConfig.getConfigurationSection("loot")!!.getKeys(false)) {
//                            val oid = configService.lootConfig.getInt("loot.$i.item").toString() + ""
//                            println("loot.$i.item")
//                            println(oid + ": " + list.contains(oid))
//                            if (list.contains(oid)) {
//                                configService.lootConfig["loot.$i.item"] = config.getString("items.$oid")
//                            } else println("ERROR: $oid")
//                        }
//                        try {
//                            configService.lootConfig.save(this.lootYML)
//                        } catch (ignored: IOException) {
//                        }
//                        sender.sendMessage("§eLoot Fixed!")
                    } else if ((args.size == 1) && (args[0].equals("reload", ignoreCase = true))) {
                        configService.reloadLoot()
                        sender.sendMessage("§eConfig reloaded!")
                    } else if (args[0] == "mobList") {
                        sender.sendMessage("§6Mob List:")
                        for (et in EntityType.entries) if (et.getName() != null) sender.sendMessage("§e" + et.getName())
                        return true
                    } else if ((args.size == 1) && (args[0].equals("error", ignoreCase = true))) {
                        errorList.add(player)
                        sender.sendMessage("§eClick on a mob to send an error report about it.")
                    } else if ((args.size == 1) && (args[0].equals("info", ignoreCase = true))) {
                        sender.sendMessage("§eMounts: " + infernalMobService.mountList.size)
                        sender.sendMessage("§eLoops: " + this.loops)
                        sender.sendMessage("§eInfernals: " + infernalMobService.infernalList.size)
                    } else if ((args.size == 1) && (args[0].equals("worldInfo", ignoreCase = true))) {
                        val enWorldList = configService.mainConfig.mobWorlds
                        val world = player.world
                        var enabled = "is not"
                        if (enWorldList.contains(world.name) || enWorldList.contains("<all>")) {
                            enabled = "is"
                        }
                        sender.sendMessage("The world you are currently in, $world $enabled enabled.")
                        sender.sendMessage("All the worlds that are enabled are: $enWorldList")
                    } else if ((args.size == 1) && (args[0].equals("help", ignoreCase = true))) {
                        throwError(sender)
                    } else if ((args.size == 1) && (args[0].equals("getloot", ignoreCase = true))) {
                        val min = configService.mainConfig.minpowers
                        val max = configService.mainConfig.maxpowers
                        val powers = MathUtils.rand(min, max)
                        val gottenLoot = lootHandler.getRandomLoot(player!!, randomMob, powers)
                        if (gottenLoot != null) {
                            player.inventory.addItem(gottenLoot)
                        }
                        sender.sendMessage("§eGave you some random loot!")
                    } else if ((args.size == 2) && (args[0].equals("getloot", ignoreCase = true))) {
                        try {
                            val index = args[1].toInt()
                            val i = lootHandler.getLoot(player, index)
                            if (i != null) {
                                player.inventory.addItem(i)
                                sender.sendMessage("§eGave you the loot at index §9$index")
                                return true
                            }
                        } catch (ignored: Exception) {
                        }
                        sender.sendMessage("§cUnable to get that loot!")
                    } else if ((args.size == 3) && (args[0].equals("giveloot", ignoreCase = true))) {
                        try {
                            val p = plugin.server.getPlayer(args[1])
                            if (p != null) {
                                val index = args[2].toInt()
                                val i = lootHandler.getLoot(p, index)
                                if (i != null) {
                                    p.inventory.addItem(i)
                                    sender.sendMessage("§eGave the player the loot at index §9$index")
                                    return true
                                }
                            } else {
                                sender.sendMessage("§cPlayer not found!!")
                                return true
                            }
                        } catch (ignored: Exception) {
                        }
                        sender.sendMessage("§cUnable to get that loot!")
                    } else if (((args.size == 2) && (args[0].equals(
                            "spawn",
                            ignoreCase = true
                        ))) || ((args[0].equals("cspawn", ignoreCase = true)) && (args.size == 6))
                    ) {
                        if ((EntityType.fromName(args[1]) != null)) {
                            val exmsg: Boolean
                            val world: World?
                            val ent: Entity
                            if ((args[0].equals(
                                    "cspawn",
                                    ignoreCase = true
                                )) && (args[2] != null) && (args[3] != null) && (args[4] != null) && (args[5] != null)
                            ) {
                                if (Bukkit.getServer().getWorld(args[2]) == null) {
                                    sender.sendMessage(args[2] + " dose not exist!")
                                    return true
                                }
                                world = Bukkit.getServer().getWorld(args[2])
                                val spoint = Location(
                                    Bukkit.getServer().getWorld(
                                        args[2]
                                    ),
                                    args[3].toInt().toDouble(), args[4].toInt().toDouble(),
                                    args[5].toInt().toDouble()
                                )
                                ent = world!!.spawnEntity(
                                    spoint,
                                    EntityType.fromName(args[1])!!
                                )
                                exmsg = true
                            } else {
                                val farSpawnLoc = player.getTargetBlock(null, 200).location
                                farSpawnLoc.y += 1.0
                                ent = player.world.spawnEntity(
                                    farSpawnLoc, EntityType.fromName(
                                        args[1]
                                    )!!
                                )
                                exmsg = false
                            }
                            val abList = infernalMobHandler.getAbilitiesAmount(ent)
                            val newMob: InfernalMob
                            val id = ent.uniqueId
                            newMob = if (abList.contains("oneup")) {
                                InfernalMob(ent, id, true, abList, 2, infernalMobService.effect)
                            } else {
                                InfernalMob(ent, id, true, abList, 1, infernalMobService.effect)
                            }

                            if (abList.contains("flying")) {
                                EntityUtils.makeFly(ent)
                            }
                            infernalMobService.infernalList.add(newMob)
                            gui!!.setName(ent)

                            infernalMobService.giveMobGear(ent, false)
                            infernalMobService.addHealth(ent, abList)
                            if (!exmsg) {
                                sender.sendMessage("Spawned a " + args[1])
                            } else if (sender is Player) {
                                sender.sendMessage("Spawned a " + args[1] + " in " + args[2] + " at " + args[3] + ", " + args[4] + ", " + args[5])
                            }
                        } else {
                            sender.sendMessage("Can't spawn a " + args[1] + "!")
                            return true
                        }
                    } else if (((args.size >= 3) && (args[0].equals(
                            "spawn",
                            ignoreCase = true
                        ))) || ((args[0].equals(
                            "cspawn",
                            ignoreCase = true
                        )) && (args.size >= 6)) || ((args[0].equals("pspawn", ignoreCase = true)) && (args.size >= 3))
                    ) {
                        if (args[0].equals("spawn", ignoreCase = true)) {
                            if ((EntityType.fromName(args[1]) != null)) {
                                val farSpawnLoc = player!!.getTargetBlock(null, 200).location
                                farSpawnLoc.y += 1.0
                                val ent = player.world.spawnEntity(
                                    farSpawnLoc, EntityType.fromName(
                                        args[1]
                                    )!!
                                )
                                val spesificAbList = mutableListOf<String>()
                                for (i in 0..(args.size - 3)) {
                                    if (Abilitiy.entries.map { it.value }.contains(args[(i + 2)])) {
                                        spesificAbList.add(args[(i + 2)])
                                    } else {
                                        sender.sendMessage(args[(i + 2)] + " is not a valid ability!")
                                        return true
                                    }
                                }
                                val newMob: InfernalMob
                                val id = ent.uniqueId
                                newMob = if (spesificAbList.contains("oneup")) {
                                    InfernalMob(ent, id, true, spesificAbList, 2, infernalMobService.effect)
                                } else {
                                    InfernalMob(ent, id, true, spesificAbList, 1, infernalMobService.effect)
                                }
                                if (spesificAbList.contains("flying")) {
                                    EntityUtils.makeFly(ent)
                                }
                                infernalMobService.infernalList.add(newMob)
                                gui.setName(ent)
                                infernalMobService.giveMobGear(ent, false)

                                infernalMobService.addHealth(ent, spesificAbList)

                                sender.sendMessage("Spawned a " + args[1] + " with the abilities:")
                                sender.sendMessage(spesificAbList.toString())
                            } else {
                                sender.sendMessage("Can't spawn a " + args[1] + "!")
                            }
                        } else if (args[0].equals("cspawn", ignoreCase = true)) {
                            //cspawn <mob> <world> <x> <y> <z> <ability> <ability>
                            if (Bukkit.getServer().getWorld(args[2]) == null) {
                                sender.sendMessage(args[2] + " dose not exist!")
                                return true
                            }
                            val world = Bukkit.getServer().getWorld(args[2])
                            val spoint = Location(
                                world,
                                args[3].toInt().toDouble(), args[4].toInt().toDouble(),
                                args[5].toInt().toDouble()
                            )
                            val abList = listOf(*args).subList(6, args.size)
                            if (cSpawn(sender, args[1], spoint, abList)) {
                                sender.sendMessage("Spawned a " + args[1] + " in " + args[2] + " at " + args[3] + ", " + args[4] + ", " + args[5] + " with the abilities:")
                                sender.sendMessage(abList.toString())
                            }
                        } else {
                            //pspawn <mob> <player> <ability> <ability>
                            val p = plugin.server.getPlayer(args[2])
                            if (p == null) {
                                sender.sendMessage(args[2] + " is not online!")
                                return true
                            }
                            val abList = listOf(*args).subList(3, args.size)
                            if (cSpawn(sender, args[1], p.location, abList)) {
                                sender.sendMessage("Spawned a " + args[1] + " at " + p.name + " with the abilities:")
                                sender.sendMessage(abList.toString())
                            }
                        }
                    } else if ((args.size == 1) && (args[0].equals("abilities", ignoreCase = true))) {
                        sender.sendMessage("--Infernal Mobs Abilities--")
                        sender.sendMessage("mama, molten, weakness, vengeance, webber, storm, sprint, lifesteal, ghastly, ender, cloaked, berserk, oneup, sapper, rust, bullwark, quicksand, thief, tosser, withering, blinding, armoured, poisonous, potions, explode, gravity, archer, necromancer, firework, flying, mounted, morph, ghost, confusing")
                    } else {
                        val oldMobAbilityList: List<String?>?
                        if ((args.size == 1) && (args[0].equals("showAbilities", ignoreCase = true))) {
                            if (PlayerUtils.getTarget(player) != null) {
                                val targeted = PlayerUtils.getTarget(player)
                                val mobId = targeted!!.uniqueId
                                if (infernalMobService.idSearch(mobId) != -1) {
                                    oldMobAbilityList = infernalMobService.findMobAbilities(mobId)
                                    if (!targeted.isDead) {
                                        sender.sendMessage("--Targeted InfernalMob's Abilities--")
                                        sender.sendMessage(oldMobAbilityList.toString())
                                    }
                                } else {
                                    sender.sendMessage("§cThis " + targeted.type.getName() + " §cis not an infernal mob!")
                                }
                            } else {
                                sender.sendMessage("§cUnable to find mob!")
                            }
                        } else if ((args[0].equals("setInfernal", ignoreCase = true)) && (args.size == 2)) {
                            if (player!!.getTargetBlock(null, 25).type == Material.SPAWNER) {
                                val delay = args[1].toInt()

                                val name = LocationUtils.getLocationName(player.getTargetBlock(null, 25).location)

                                configService.mobSaveConfig["infernalSpanwers.$name"] = delay
                                configService.saveSave()
                                sender.sendMessage("§cSpawner set to infernal with a $delay second delay!")
                            } else {
                                sender.sendMessage("§cYou must be looking a spawner to make it infernal!")
                            }
                        } else if ((args[0].equals("kill", ignoreCase = true)) && (args.size == 2)) {
                            val size = args[1].toInt()
                            for (e in player.getNearbyEntities(size.toDouble(), size.toDouble(), size.toDouble())) {
                                val id = infernalMobService.idSearch(e.uniqueId)
                                if (id != -1) {
                                    removeMob(id)
                                    e.remove()
                                    plugin.logger.log(Level.INFO, "Entity remove due to /kill")
                                }
                            }
                            sender.sendMessage("§eKilled all infernal mobs near you!")
                        } else if ((args[0].equals(
                                "killall",
                                ignoreCase = true
                            )) && (args.size == 1 || args.size == 2)
                        ) {
                            var w: World? = null
                            if (args.size == 1 && sender is Player) {
                                w = sender.world
                            } else if (args.size == 2) {
                                w = plugin.server.getWorld(args[1])
                            }

                            if (w != null) {
                                for (e in w.entities) {
                                    val id = infernalMobService.idSearch(e.uniqueId)
                                    if (id != -1) {
                                        removeMob(id)
                                        if (e is LivingEntity) {
                                            e.customName = null
                                        }
                                        plugin.logger.log(Level.INFO, "Entity remove due to /killall")
                                        e.remove()
                                    }
                                }
                                sender.sendMessage("§eKilled all loaded infernal mobs in that world!")
                            } else {
                                sender.sendMessage("§cWorld not found!")
                            }
                        } else if (args[0].equals("mobs", ignoreCase = true)) {
                            sender.sendMessage("§6List of Mobs:")
                            for (e in EntityType.values()) if (e != null) sender.sendMessage(e.toString())
                        } else if (args[0].equals("setloot", ignoreCase = true)) {
                            configService.setItem(
                                player.inventory.itemInMainHand,
                                "loot." + args[1],
                                configService.lootConfig
                            )
                            sender.sendMessage("§eSet loot at index " + args[1] + " §eto item in hand.")
                        } else {
                            throwError(sender)
                        }
                    }
                } else {
                    sender.sendMessage("§cYou don't have permission to use this command!")
                }
            } catch (x: Exception) {
                throwError(sender)
                x.printStackTrace()
            }
        }
        return true
    }

    private fun removeMob(id: Any) {

    }

    override fun onTabComplete(sender: CommandSender, cmd: Command, label: String, args: Array<String>): List<String>? {
        val allAbilitiesList: List<String> = ArrayList(
            mutableListOf(
                "confusing",
                "ghost",
                "morph",
                "mounted",
                "flying",
                "gravity",
                "firework",
                "necromancer",
                "archer",
                "molten",
                "mama",
                "potions",
                "explode",
                "berserk",
                "weakness",
                "vengeance",
                "webber",
                "storm",
                "sprint",
                "lifesteal",
                "ghastly",
                "ender",
                "cloaked",
                "oneup",
                "sapper",
                "rust",
                "bullwark",
                "quicksand",
                "thief",
                "tosser",
                "withering",
                "blinding",
                "armoured",
                "poisonous"
            )
        )
        val commands: Set<String> = HashSet(
            mutableListOf(
                "reload",
                "worldInfo",
                "error",
                "getloot",
                "setloot",
                "giveloot",
                "abilities",
                "showAbilities",
                "setInfernal",
                "spawn",
                "cspawn",
                "pspawn",
                "kill",
                "killall"
            )
        )
        if (sender.hasPermission("infernal_mobs.commands")) {
            val newTab: MutableList<String> = ArrayList()
            if (args.size == 1) {
                if (args[0].isEmpty()) return ArrayList(commands)
                commands.forEach(Consumer { tab: String ->
                    if (tab.lowercase(Locale.getDefault()).startsWith(
                            args[0].lowercase(Locale.getDefault())
                        )
                    ) newTab.add(tab)
                })
            }
            if (args[0].equals("getloot", ignoreCase = true) || args[0].equals("setloot", ignoreCase = true)) {
                if (args.size == 2) {
                    newTab.add("1")
                }
            }
            if (args[0].equals("giveloot", ignoreCase = true)) {
                if (args.size == 2) {
                    newTab.addAll(
                        Bukkit.getOnlinePlayers().stream().map { obj: HumanEntity -> obj.name }.collect(
                            Collectors.toList()
                        )
                    )
                }
                if (args.size == 3) {
                    newTab.add("1")
                }
            }
            if (args[0].equals("setinfernal", ignoreCase = true)) {
                if (args.size == 2) {
                    newTab.add("10")
                }
            }
            if (args.size == 2) {
                if (args[0].equals("spawn", ignoreCase = true) || args[0].equals(
                        "cspawn",
                        ignoreCase = true
                    ) || args[0].equals("pspawn", ignoreCase = true)
                ) {
                    if (args[1].isEmpty()) newTab.addAll(
                        Arrays.stream(EntityType.values()).filter { m: EntityType -> m.isSpawnable && m.isAlive }
                            .map { obj: EntityType -> obj.name }.collect(
                                Collectors.toList()
                            )
                    )
                    else Arrays.stream(EntityType.values()).filter { m: EntityType -> m.isSpawnable && m.isAlive }
                        .map { obj: EntityType -> obj.name }.collect(
                            Collectors.toList()
                        ).forEach(
                            Consumer { tab: String ->
                                if (tab.lowercase(Locale.getDefault()).startsWith(
                                        args[1].lowercase(Locale.getDefault())
                                    )
                                ) newTab.add(tab)
                            })
                }
                if (args[0].equals("killall", ignoreCase = true)) {
                    if (args[args.size - 1].isEmpty()) newTab.addAll(
                        Bukkit.getWorlds().stream().map { obj: World -> obj.name }.collect(
                            Collectors.toList()
                        )
                    )
                    else Bukkit.getWorlds().stream().map { obj: World -> obj.name }.collect(Collectors.toList())
                        .forEach(
                            Consumer { tab: String ->
                                if (tab.lowercase(Locale.getDefault()).startsWith(
                                        args[args.size - 1].lowercase(Locale.getDefault())
                                    )
                                ) newTab.add(tab)
                            })
                }
                if (args[0].equals("kill", ignoreCase = true)) {
                    newTab.add("1")
                }
            }
            if (args[0].equals("cspawn", ignoreCase = true)) {
                if (args.size == 3) {
                    if (args[args.size - 1].isEmpty()) newTab.addAll(
                        Bukkit.getWorlds().stream().map { obj: World -> obj.name }.collect(
                            Collectors.toList()
                        )
                    )
                    else Bukkit.getWorlds().stream().map { obj: World -> obj.name }.collect(Collectors.toList())
                        .forEach(
                            Consumer { tab: String ->
                                if (tab.lowercase(Locale.getDefault()).startsWith(
                                        args[args.size - 1].lowercase(Locale.getDefault())
                                    )
                                ) newTab.add(tab)
                            })
                }
                if (args.size in 4..6) {
                    newTab.add("~")
                }
                if (args.size >= 7) {
                    if (args[args.size - 1].isEmpty()) newTab.addAll(allAbilitiesList)
                    else allAbilitiesList.forEach(Consumer { tab: String ->
                        if (tab.lowercase(Locale.getDefault()).startsWith(
                                args[args.size - 1].lowercase(Locale.getDefault())
                            )
                        ) newTab.add(tab)
                    })
                }
            }
            if (args[0].equals("pspawn", ignoreCase = true)) {
                if (args.size == 3) {
                    if (args[args.size - 1].isEmpty()) newTab.addAll(
                        Bukkit.getOnlinePlayers().stream().map { obj: HumanEntity -> obj.name }.collect(
                            Collectors.toList()
                        )
                    )
                    else Bukkit.getOnlinePlayers().stream().map { obj: HumanEntity -> obj.name }
                        .collect(Collectors.toList()).forEach(
                            Consumer { tab: String ->
                                if (tab.lowercase(Locale.getDefault()).startsWith(
                                        args[args.size - 1].lowercase(Locale.getDefault())
                                    )
                                ) newTab.add(tab)
                            })
                }
                if (args.size > 3) {
                    if (args[args.size - 1].isEmpty()) newTab.addAll(allAbilitiesList)
                    else allAbilitiesList.forEach(Consumer { tab: String ->
                        if (tab.lowercase(Locale.getDefault()).startsWith(
                                args[args.size - 1].lowercase(Locale.getDefault())
                            )
                        ) newTab.add(tab)
                    })
                }
            }
            if (args.size >= 3) {
                if (args[0].equals("spawn", ignoreCase = true)) {
                    if (args[args.size - 1].isEmpty()) newTab.addAll(allAbilitiesList)
                    else allAbilitiesList.forEach(Consumer { tab: String ->
                        if (tab.lowercase(Locale.getDefault()).startsWith(
                                args[args.size - 1].lowercase(Locale.getDefault())
                            )
                        ) newTab.add(tab)
                    })
                }
            }
            return newTab
        }
        return null
    }

    private fun throwError(sender: CommandSender) {
//        sender.sendMessage("--Infernal Mobs v" + description.version + "--")
        sender.sendMessage("Usage: /im reload")
        sender.sendMessage("Usage: /im worldInfo")
        sender.sendMessage("Usage: /im error")
        sender.sendMessage("Usage: /im getloot <index>")
        sender.sendMessage("Usage: /im setloot <index>")
        sender.sendMessage("Usage: /im giveloot <player> <index>")
        sender.sendMessage("Usage: /im abilities")
        sender.sendMessage("Usage: /im showAbilities")
        sender.sendMessage("Usage: /im setInfernal <time delay>")
        sender.sendMessage("Usage: /im spawn <mob> <ability> <ability>")
        sender.sendMessage("Usage: /im cspawn <mob> <world> <x> <y> <z> <ability> <ability>")
        sender.sendMessage("Usage: /im pspawn <mob> <player> <ability> <ability>")
        sender.sendMessage("Usage: /im kill <size>")
        sender.sendMessage("Usage: /im killall <world>")
    }

    private fun cSpawn(sender: CommandSender, mob: String, l: Location, abList: List<String>): Boolean {
        //cspawn <mob> <world> <x> <y> <z> <ability> <ability>
        if ((EntityType.fromName(mob) != null)) {
            val ent = l.world.spawnEntity(
                l,
                EntityType.fromName(mob)!!
            )
            val newMob: InfernalMob
            val id = ent.uniqueId
            newMob = if (abList.contains("oneup")) {
                InfernalMob(ent, id, true, abList, 2, infernalMobService.effect)
            } else {
                InfernalMob(ent, id, true, abList, 1, infernalMobService.effect)
            }
            if (abList.contains("flying")) {
                EntityUtils.makeFly(ent)
            }
            infernalMobService.infernalList.add(newMob)
            gui!!.setName(ent)

            infernalMobService.giveMobGear(ent, false)
            infernalMobService.addHealth(ent, abList)
            return true
        } else {
            sender.sendMessage("Can't spawn a $mob!")
            return false
        }
    }

}