package io.github.thehrz.infernalmobs.gui

import io.github.thehrz.infernalmobs.InfernalMobs
import io.github.thehrz.infernalmobs.handler.InfernalMobHandler
import io.github.thehrz.infernalmobs.services.ConfigService
import io.github.thehrz.infernalmobs.services.InfernalMobService
import io.github.thehrz.infernalmobs.utils.StringUtils
import org.bukkit.Bukkit
import org.bukkit.ChatColor
import org.bukkit.boss.BarColor
import org.bukkit.boss.BarFlag
import org.bukkit.boss.BarStyle
import org.bukkit.boss.BossBar
import org.bukkit.entity.*
import org.bukkit.event.Listener
import org.bukkit.scoreboard.DisplaySlot
import org.bukkit.scoreboard.Objective
import org.bukkit.scoreboard.Scoreboard
import org.koin.core.annotation.Single
import java.io.IOException
import java.util.*

@Single
class GUI internal constructor(
    private val plugin: InfernalMobs,
    private val infernalMobService: InfernalMobService,
    private val infernalMobHandler: InfernalMobHandler,
    private val configService: ConfigService
) : Listener {
    fun setName(ent: Entity) {
        try {
            //System.out.println("SN1 " + ent);
            if (configService.mainConfig.nameTags.level != 0) {
                val tittle = getMobNameTag(ent)
                ent.customName = tittle
                if (configService.mainConfig.nameTags.level == 2) {
                    ent.isCustomNameVisible = true
                }
            }
        } catch (x: Exception) {
            println("Error in setName: ")
            x.printStackTrace()
        }
    }

    fun getMobNameTag(entity: Entity): String {
        val oldMobAbilityList: List<String?> = infernalMobService.findMobAbilities(entity.uniqueId)
        var tittle: String? = null

        tittle = configService.mainConfig.nameTags.name

        val mobName = entity.type.getName()!!.replace("_", " ")

        tittle = tittle.replace(
            "<mobName>",
            mobName.substring(0, 1).uppercase(Locale.getDefault()) + mobName.substring(1)
        )
        tittle = tittle.replace("<mobLevel>", "" + oldMobAbilityList.size)
        var abilities: String
        var count = 4
        do {
            abilities = StringUtils.generateString(count, oldMobAbilityList)
            count--
        } while ((tittle.length + abilities.length + mobName.length) > 64)
        tittle = tittle.replace(
            "<abilities>",
            abilities.substring(0, 1).uppercase(Locale.getDefault()) + abilities.substring(1)
        )
        //Prefix
        var prefix: String = configService.mainConfig.namePrefix
        //todo: levelPrefixs
//        if (plugin.config.getString("levelPrefixs." + oldMobAbilityList.size) != null)
//            prefix = plugin.getConfig().getString("levelPrefixs." + oldMobAbilityList.size)
        tittle = tittle.replace("<prefix>", prefix.substring(0, 1).uppercase(Locale.getDefault()) + prefix.substring(1))
        tittle = ChatColor.translateAlternateColorCodes('&', tittle)

        return tittle
    }

    private val playerScoreBoard = HashMap<String, Scoreboard?>()
    private val bossBars = HashMap<Entity, Any>()


    fun fixBar(p: Player) {
        //System.out.println("fixBar");
        val b = infernalMobService.getNearbyBoss(p)
        if (b != null) {
            //System.out.println("Dead: " + b.isDead());
            //System.out.println("HP: " + ((Damageable)b).getHealth());
            if (b.isDead || (b as Damageable).health <= 0) {
                if (configService.mainConfig.bossBarSettings.enable) {
                    try {
                        for (p2 in (bossBars[b] as BossBar).players) (bossBars[b] as BossBar).removePlayer(p2)
                        bossBars.remove(b)
                    } catch (x: Exception) {
                    }
                }
                val mobIndex: Int = infernalMobService.idSearch(b.uniqueId)
                try {
                    if (mobIndex != -1) infernalMobHandler.removeMob(mobIndex)
                } catch (e: IOException) {
                }
                clearInfo(p)
            } else {
                if (configService.mainConfig.bossBarSettings.enable) {
                    showBossBar(p, b)
                }
                if (configService.mainConfig.scoreBoard.enable) {
                    fixScoreboard(p, b, infernalMobService.findMobAbilities(b.getUniqueId()))
                }
            }
        } else clearInfo(p)
    }

    private fun showBossBar(p: Player, e: Entity) {
        val oldMobAbilityList: List<String?> = infernalMobService.findMobAbilities(e.uniqueId)
        var tittle: String = configService.mainConfig.bossBarSettings.name
        var mobName = e.type.getName()!!.replace("_", " ")
        if (e.type == EntityType.SKELETON) {
            val sk = e as Skeleton
            if (sk.skeletonType == Skeleton.SkeletonType.WITHER) {
                mobName = "WitherSkeleton"
            }
        }
        var prefix: String = configService.mainConfig.namePrefix
        //TODO: levelPrefixs
//        if (plugin.getConfig().getString("levelPrefixs." + oldMobAbilityList.size) != null) {
//            prefix = plugin.getConfig().getString("levelPrefixs." + oldMobAbilityList.size)
//        }
        tittle =
            tittle.replace("<prefix>", prefix.substring(0, 1).uppercase(Locale.getDefault()) + prefix.substring(1))
        tittle = tittle.replace(
            "<mobName>",
            mobName.substring(0, 1).uppercase(Locale.getDefault()) + mobName.substring(1)
        )
        tittle = tittle.replace("<mobLevel>", oldMobAbilityList.size.toString() + "")
        var abilities: String = StringUtils.generateString(5, oldMobAbilityList)
        var count = 4
        try {
            do {
                abilities = StringUtils.generateString(count, oldMobAbilityList)
                count--
                if (count <= 0) {
                    break
                }
            } while (tittle.length + abilities.length + mobName.length > 64)
        } catch (x: Exception) {
            println("showBossBar error: ")
            x.printStackTrace()
        }
        tittle = tittle.replace(
            "<abilities>",
            abilities.substring(0, 1).uppercase(Locale.getDefault()) + abilities.substring(1)
        )
        tittle = ChatColor.translateAlternateColorCodes('&', tittle)

        if (!bossBars.containsKey(e)) {
            var bc = BarColor.valueOf(configService.mainConfig.bossBarSettings.color)
            var bs = BarStyle.valueOf(configService.mainConfig.bossBarSettings.style)

            //TODO:Per Level Setings
//            val lc: String = plugin.getConfig()
//                .getString("bossBarSettings.perLevel." + oldMobAbilityList.size + ".color")
//            if (lc != null) bc = BarColor.valueOf(lc)
//            val ls: String = plugin.getConfig()
//                .getString("bossBarSettings.perLevel." + oldMobAbilityList.size + ".style")
//            if (ls != null) bs = BarStyle.valueOf(ls)
            //TODO:Per InfernalMob Setings
//            val mc: String =
//                plugin.getConfig().getString("bossBarSettings.perMob." + e.type.getName() + ".color")
//            if (mc != null) bc = BarColor.valueOf(mc)
//            val ms: String =
//                plugin.getConfig().getString("bossBarSettings.perMob." + e.type.getName() + ".style")
//            if (ms != null) bs = BarStyle.valueOf(ms)
            val bar = Bukkit.createBossBar(tittle, bc, bs, BarFlag.CREATE_FOG)
            bar.isVisible = true
            bossBars[e] = bar
        }
        if (!(bossBars[e] as BossBar).players.contains(p)) (bossBars[e] as BossBar).addPlayer(p)
        val health = (e as Damageable).health.toFloat()
        val maxHealth = e.maxHealth.toFloat()
        val setHealth = (health * 100.0f) / maxHealth
        (bossBars[e] as BossBar).progress = (setHealth / 100.0f).toDouble()
    }

    private fun clearInfo(player: Player) {
        if (configService.mainConfig.bossBarSettings.enable) {
            //BossBarAPI.removeBar(player);
            for ((_, value) in bossBars) if ((value as BossBar).players.contains(player)) value.removePlayer(player)
        }
        if (configService.mainConfig.scoreBoard.enable) {
            try {
                player.scoreboard.resetScores(player)
                player.scoreboard.getObjective(DisplaySlot.SIDEBAR)!!.unregister()
            } catch (_: Exception) {
            }
        }
    }

    private fun fixScoreboard(player: Player, e: Entity, abilityList: List<String?>) {
        if (configService.mainConfig.scoreBoard.enable && (e is Damageable)) {
            //String name = getMobNameTag(e);
            //Get Display
            //System.out.println("sb1");
            if (playerScoreBoard[player.name] == null) {
                //System.out.println("Creating ScoreBoard");
                val manager = Bukkit.getScoreboardManager()
                val board = manager.newScoreboard
                playerScoreBoard[player.name] = board
            }
            //System.out.println("sb2");
            val o: Objective
            val board = playerScoreBoard[player.name]
            //System.out.println("Board = " + board);
            if (board!!.getObjective(DisplaySlot.SIDEBAR) == null) {
                o = board.registerNewObjective(player.name, "dummy")
                o.displaySlot = DisplaySlot.SIDEBAR
            } else {
                o = board.getObjective(DisplaySlot.SIDEBAR)!!
            }
            //System.out.println("sb3");
            //Name
            //System.out.println("Name: " + e.getType().getName());
            //System.out.println("OBJ = " + o);
            o.displayName = e.getType().getName()!!
            //System.out.println("Set ScoreBoard Name");
            //Remove Old
            //for(OfflinePlayer p : board.getPlayers())
            //    board.resetScores(p);
            for (s in board.entries) board.resetScores(s)
            //Power Display
            var score = 1
            //System.out.println("sb4");
            for (ability in abilityList) {
                //Score pointsDisplayScore = o.getScore(Bukkit.getOfflinePlayer("§r" + ability));
                //pointsDisplayScore.setScore(score);
                o.getScore("§r$ability").score = score
                score += 1
            }
            //Score abDisplayScore = o.getScore(Bukkit.getOfflinePlayer("§e§lAbilities:"));
            //abDisplayScore.setScore(score);
            o.getScore("§e§lAbilities:").score = score
            //Health
            //System.out.println("sb5");
            if (configService.mainConfig.scoreBoard.showHealth) {
                //System.out.println("shosb");
                //Display HP
                score += 1
                val health = e.health.toFloat()
                val maxHealth = e.maxHealth.toFloat()
                val roundOff = Math.round(health * 100.0) / 100.0
                //Score hDisplayScore = o.getScore(Bukkit.getOfflinePlayer(roundOff + "/" + maxHealth));
                //hDisplayScore.setScore(score);
                o.getScore("$roundOff/$maxHealth").score = score
                score += 1
                //Score htDisplayScore = o.getScore(Bukkit.getOfflinePlayer("§e§lHealth:"));
                //htDisplayScore.setScore(score);
                o.getScore("§e§lHealth:").score = score
            }
            //System.out.println("sb6");
            //Display
            if ((player.scoreboard == null) || (player.scoreboard.getObjective(DisplaySlot.SIDEBAR) == null) || (player.scoreboard
                    .getObjective(DisplaySlot.SIDEBAR)!!
                    .name == null) || (player.scoreboard
                    .getObjective(DisplaySlot.SIDEBAR)!!
                    .name != board.getObjective(DisplaySlot.SIDEBAR)!!.name)
            ) {
                //System.out.println("Set SB");
                player.scoreboard = board
            }
            //System.out.println("sb7");
        }
    }

}