package io.github.thehrz.infernalmobs.handler

import io.github.thehrz.infernalmobs.InfernalMobs
import io.github.thehrz.infernalmobs.models.LevelledEnchantment
import io.github.thehrz.infernalmobs.services.ConfigService
import io.github.thehrz.infernalmobs.utils.ItemUtils
import io.github.thehrz.infernalmobs.utils.MathUtils
import io.github.thehrz.infernalmobs.utils.StringUtils
import org.bukkit.*
import org.bukkit.block.Banner
import org.bukkit.block.banner.Pattern
import org.bukkit.enchantments.Enchantment
import org.bukkit.entity.Player
import org.bukkit.inventory.ItemStack
import org.bukkit.inventory.meta.*
import org.bukkit.potion.PotionData
import org.bukkit.potion.PotionType
import org.koin.core.annotation.Single
import java.util.*
import java.util.logging.Level

@Single
class LootHandler(val plugin: InfernalMobs, val configService: ConfigService) {
    fun getRandomLoot(player: Player, mob: String?, powers: Int): ItemStack {
        val lootList = mutableListOf<Int>()
        for (i in configService.lootConfig.getConfigurationSection("loot")?.getKeys(false)
            ?: return ItemStack(Material.AIR)) {
            if ((configService.lootConfig.getString("loot.$i") != null) &&
                ((configService.lootConfig.getList("loot.$i.mobs") == null) ||
                        (configService.lootConfig.getList("loot.$i.mobs", ArrayList<Any>())!!.contains(mob))) &&
                (configService.lootConfig.getString("loot.$i.chancePercentage") == null ||
                        MathUtils.rand(1, 100) <= configService.lootConfig.getInt("loot.$i.chancePercentage"))
            ) {
                if (mobPowerLevelFine(i.toInt(), powers)) {
                    lootList.add(i.toInt())
                }
            }
        }
        if (configService.mainConfig.debug) plugin.logger.log(
            Level.INFO,
            "Loot List $lootList"
        )
        return if (lootList.isNotEmpty()) {
            getLoot(player, lootList[MathUtils.rand(1, lootList.size) - 1])
        } else ItemStack(Material.AIR)
    }

    fun getLoot(player: Player, loot: Int): ItemStack {
        var i: ItemStack? = null
        if (configService.lootConfig.getStringList("loot.$loot.commands").isNotEmpty()) {
            val commandList = configService.lootConfig.getStringList("loot.$loot.commands")
            for (command in commandList) {
                Bukkit.getServer().dispatchCommand(
                    Bukkit.getConsoleSender(),
                    ChatColor.translateAlternateColorCodes('&', command).replace("player", player.name)
                )
            }
        }
        //        if (this.configService.lootConfig.getString("loot." + loot + ".staff.id") != null) {
        //            int id = this.configService.lootConfig.getInt("loot." + loot + ".staff.id");
        //            ArrayList<String> spells = new ArrayList();
        //            if (!this.configService.lootConfig.getStringList("loot." + loot + ".staff.spells").isEmpty()) {
        //                spells = (ArrayList) this.configService.lootConfig.getStringList("loot." + loot + ".staff.spells");
        //            }
        //            return this.wMagic.getStaffWithSpells(id, spells);
        //        }

        return getItem(loot)
    }

    fun getItem(loot: Int): ItemStack {
        //System.out.println("Get Loot: " + loot);
        val setItem = configService.lootConfig.getString("loot.$loot.item")

        val setAmountString = configService.lootConfig.getString("loot.$loot.amount")
        val setAmount = if (setAmountString != null) {
            StringUtils.getIntFromString(setAmountString)
        } else 1
        val stack = ItemStack(ItemUtils.getMaterial(setItem), setAmount)
        //Name
        var name: String? = null
        if (configService.lootConfig.getString("loot.$loot.name") != null && configService.lootConfig.isString("loot.$loot.name")) {
            name = configService.lootConfig.getString("loot.$loot.name")
            name = ItemUtils.prosessLootName(name!!, stack)
        } else if (configService.lootConfig.isList("loot.$loot.name")) {
            val names = configService.lootConfig.getStringList("loot.$loot.name")
            if (names.isNotEmpty()) {
                name = names[MathUtils.rand(1, names.size) - 1]
                name = ItemUtils.prosessLootName(name, stack)
            }
        }
        //Lore
        val loreList = mutableListOf<String>()
        for (i in 0..32) {
            if (configService.lootConfig.getString("loot.$loot.lore$i") != null) {
                var lore = configService.lootConfig.getString("loot.$loot.lore$i")
                lore = ChatColor.translateAlternateColorCodes('&', lore!!)
                loreList.add(lore)
            }
        }
        if (configService.lootConfig.getStringList("loot.$loot.lore").isNotEmpty()) {
            val l = configService.lootConfig.getStringList("loot.$loot.lore")
            var min = l.size
            if (configService.lootConfig.getString("loot.$loot.minLore") != null) min =
                configService.lootConfig.getInt("loot.$loot.minLore")
            var max = l.size
            if (configService.lootConfig.getString("loot.$loot.maxLore") != null) max =
                configService.lootConfig.getInt("loot.$loot.maxLore")
            if (l.isNotEmpty()) for (i in 0 until MathUtils.rand(min, max)) {
                val lore = l[MathUtils.rand(1, l.size) - 1]
                l.remove(lore)
                loreList.add(ItemUtils.prosessLootName(lore, stack))
            }
        }
        val meta = stack.itemMeta
        //Durability
        if (configService.lootConfig.getString("loot.$loot.durability") != null) {
            val durabilityString = configService.lootConfig.getString("loot.$loot.durability")
            val durability = StringUtils.getIntFromString(durabilityString!!)
            (meta as Damageable).damage = durability
            //stack.setDurability((short) durability);
        }
        if (name != null) {
            meta.setDisplayName(name)
        }
        if (loreList.isNotEmpty()) {
            meta.lore = loreList
        }
        if (meta != null) {
            stack.setItemMeta(meta)
        }
        //Colour
        if (configService.lootConfig.getString("loot.$loot.colour") != null && stack.type.toString()
                .lowercase(Locale.getDefault())
                .contains("leather")
        ) {
            val c = configService.lootConfig.getString("loot.$loot.colour")
            val split = c!!.split(",".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
            val colour = Color.fromRGB(split[0].toInt(), split[1].toInt(), split[2].toInt())
            ItemUtils.dye(stack, colour)
        }
        //Book
        if ((stack.type == Material.WRITTEN_BOOK) || (stack.type == Material.WRITABLE_BOOK)) {
            val bMeta = stack.itemMeta as BookMeta
            if (configService.lootConfig.getString("loot.$loot.author") != null) {
                var author = configService.lootConfig.getString("loot.$loot.author")
                author = ChatColor.translateAlternateColorCodes('&', author!!)
                bMeta.author = author
            }
            if (configService.lootConfig.getString("loot.$loot.title") != null) {
                var title = configService.lootConfig.getString("loot.$loot.title")
                title = ChatColor.translateAlternateColorCodes('&', title!!)
                bMeta.setTitle(title)
            }
            if (configService.lootConfig.getString("loot.$loot.pages") != null) {
                for (i in configService.lootConfig.getConfigurationSection("loot.$loot.pages")!!
                    .getKeys(false)) {
                    var page = configService.lootConfig.getString("loot.$loot.pages.$i")
                    page = ChatColor.translateAlternateColorCodes('&', page!!)
                    bMeta.addPage(page)
                }
            }
            stack.setItemMeta(bMeta)
        }
        //Banners
        if (stack.type.toString().contains("BANNER")) {
            val b = stack.itemMeta as BannerMeta
            val patList = configService.lootConfig.getList("loot.$loot.patterns") as List<Pattern>?
            if (!patList.isNullOrEmpty()) b.patterns = patList
            stack.setItemMeta(b)
        }
        //Shield
        if (stack.type == Material.SHIELD) {
            val im = stack.itemMeta
            val bmeta = im as BlockStateMeta

            val b = bmeta.blockState as Banner
            val patList = configService.lootConfig.getList("loot.$loot.patterns") as List<Pattern>?
            b.baseColor = DyeColor.valueOf(configService.lootConfig.getString("loot.$loot.colour")!!)
            b.patterns = patList!!
            b.update()
            bmeta.blockState = b
            stack.setItemMeta(bmeta)
        }
        //Owner
        if (stack.type == Material.PLAYER_HEAD) {
            val owner = configService.lootConfig.getString("loot.$loot.owner")
            val sm = stack.itemMeta as SkullMeta
            sm.setOwningPlayer(Bukkit.getOfflinePlayer(UUID.fromString(owner)))
            stack.setItemMeta(sm)
        }
        //Potions
        if (configService.lootConfig.getString("loot.$loot.potion") != null) if (stack.type == Material.POTION || stack.type == Material.SPLASH_POTION || stack.type == Material.LINGERING_POTION) {
            val pMeta = stack.itemMeta as PotionMeta
            val pn = configService.lootConfig.getString("loot.$loot.potion")
            pMeta.basePotionData =
                PotionData(PotionType.valueOf(pn!!), false, false)
            stack.setItemMeta(pMeta)
        }
        var enchAmount = 0
        for (e in 0..10) {
            if (configService.lootConfig.getString("loot.$loot.enchantments.$e") != null) {
                enchAmount++
            }
        }
        //System.out.println("Enchantments Found: " + enchAmount);
        if (enchAmount > 0) {
            var enMin = enchAmount / 2
            if (enMin < 1) {
                enMin = 1
            }
            var enMax = enchAmount
            if ((configService.lootConfig.getString("loot.$loot.minEnchantments") != null) && (configService.lootConfig.getString(
                    "loot.$loot.maxEnchantments"
                ) != null)
            ) {
                enMin = configService.lootConfig.getInt("loot.$loot.minEnchantments")
                enMax = configService.lootConfig.getInt("loot.$loot.maxEnchantments")
            }
            //int enchNeeded = new Random().nextInt(enMax + 1 - enMin) + enMin;
            val enchNeeded = MathUtils.rand(enMin, enMax)
            //System.out.println("Enchantments Needed: " + enchNeeded);
            val enchList = mutableListOf<LevelledEnchantment>()
            var safety = 0
            var j = 0
            var chance: Int
            do {
                if (configService.lootConfig.getString("loot.$loot.enchantments.$j") != null) {
                    var enChance = 1
                    if (configService.lootConfig.getString("loot.$loot.enchantments.$j.chance") != null) {
                        enChance = configService.lootConfig.getInt("loot.$loot.enchantments.$j.chance")
                    }
                    chance = Random().nextInt(enChance - 1 + 1) + 1
                    if (chance == 1) {
                        val enchantment =
                            configService.lootConfig.getString("loot.$loot.enchantments.$j.enchantment")!!
                                .lowercase(Locale.getDefault())
                        val levelString = configService.lootConfig.getString("loot.$loot.enchantments.$j.level")
                        var level = StringUtils.getIntFromString(levelString!!)
                        //System.out.print("1: " + NamespacedKey.minecraft(enchantment));
                        //System.out.print("2: " + Enchantment.getByKey(NamespacedKey.minecraft(enchantment)));
                        if (Enchantment.getByKey(NamespacedKey.minecraft(enchantment)) != null) {
                            if (level < 1) {
                                level = 1
                            }
                            val le = LevelledEnchantment(
                                Enchantment.getByKey(NamespacedKey.minecraft(enchantment)),
                                level
                            )

                            var con = false
                            for (testE in enchList) {
                                if (testE.getEnchantment == le.getEnchantment) {
                                    con = true
                                }
                            }
                            if (!con) {
                                enchList.add(le)
                            }
                        } else {
                            println("Error: No valid drops found!")
                            println("Error: $enchantment is not a valid enchantment!")
                            return ItemStack(Material.AIR)
                        }
                    }
                }
                j++
                if (j > enchAmount) {
                    j = 0
                    safety++
                }
                if (safety >= enchAmount * 100) {
                    //System.out.println("Error: No valid drops found!");
                    //System.out.println("Error: Please increase chance for enchantments on item " + loot);
                    //return null;
                    break
                }
            } while (enchList.size != enchNeeded)
            for (le in enchList) {
                if (stack.type == Material.ENCHANTED_BOOK) {
                    val enchantMeta = stack.itemMeta as EnchantmentStorageMeta
                    enchantMeta.addStoredEnchant(le.getEnchantment!!, le.getLevel, true)
                    stack.setItemMeta(enchantMeta)
                } else {
                    stack.addUnsafeEnchantment(le.getEnchantment!!, le.getLevel)
                }
            }
        }
        return stack
    }

    private fun mobPowerLevelFine(lootId: Int, mobPowers: Int): Boolean {
        var min = 0
        var max = 99
        if (configService.lootConfig.getString("loot.$lootId.powersMin") != null) {
            min = configService.lootConfig.getInt("loot.$lootId.powersMin")
        }
        if (configService.lootConfig.getString("loot.$lootId.powersMax") != null) max =
            configService.lootConfig.getInt("loot.$lootId.powersMax")
        if (configService.mainConfig.debug) {
            plugin.logger.log(
                Level.INFO,
                "Loot $lootId min = $min and max = $max"
            )
        }
        return (mobPowers >= min) && (mobPowers <= max)
    }
}