package io.github.thehrz.infernalmobs.handler

import io.github.thehrz.infernalmobs.InfernalMobs
import io.github.thehrz.infernalmobs.gui.GUI
import io.github.thehrz.infernalmobs.models.InfernalMob
import io.github.thehrz.infernalmobs.services.ConfigService
import io.github.thehrz.infernalmobs.services.InfernalMobService
import io.github.thehrz.infernalmobs.tasks.ArrowHomingTask
import io.github.thehrz.infernalmobs.utils.BlockUtils
import io.github.thehrz.infernalmobs.utils.EntityUtils
import io.github.thehrz.infernalmobs.utils.LocationUtils
import org.bukkit.Color
import org.bukkit.GameMode
import org.bukkit.Location
import org.bukkit.Material
import org.bukkit.attribute.Attribute
import org.bukkit.entity.*
import org.bukkit.inventory.ItemStack
import org.bukkit.inventory.meta.Damageable
import org.bukkit.inventory.meta.LeatherArmorMeta
import org.bukkit.inventory.meta.PotionMeta
import org.bukkit.potion.PotionEffect
import org.bukkit.potion.PotionEffectType
import org.bukkit.util.Vector
import org.koin.core.annotation.Single
import java.util.*
import java.util.logging.Level

@Single
class MagicHandler(
    val configService: ConfigService,
    val infernalMobHandler: InfernalMobHandler,
    val infernalMobService: InfernalMobService,
    val plugin: InfernalMobs,
    val gui: GUI
) {
    fun doMagic(vic: Entity, atc: Entity, playerIsVictom: Boolean, ability: String, id: UUID) {
        val min = 1
        val max = 10
        var randomNum = Random().nextInt(max - min) + min
        if ((atc is Player)) {
            randomNum = 1
        }
        try {
            if ((atc is Player)) {
                when (ability) {
                    "tosser" -> if ((vic !is Player) || ((!vic.isSneaking) && (vic.gameMode != GameMode.CREATIVE))) {
                        vic.velocity = atc.getLocation().toVector().subtract(vic.location.toVector())
                    }

                    "gravity" -> if ((vic !is Player) || ((!vic.isSneaking) && (vic.gameMode != GameMode.CREATIVE))) {
                        val feetBlock = vic.location
                        feetBlock.y -= 2.0
                        val block = feetBlock.world.getBlockAt(feetBlock)
                        if (block.type != Material.AIR) {
                            EntityUtils.levitate(
                                vic, configService.mainConfig.gravityLevitateLength
                            )
                        }
                    }

                    "ghastly", "necromancer" -> if ((!vic.isDead) && ((vic !is Player) || ((!vic.isSneaking) && (vic.gameMode != GameMode.CREATIVE)))) {
                        val fb = if (ability == "ghastly") {
                            (atc as LivingEntity).launchProjectile(
                                Fireball::class.java
                            )
                        } else {
                            (atc as LivingEntity).launchProjectile(
                                WitherSkull::class.java
                            )
                        }
                        EntityUtils.moveToward(fb, vic.location, 0.6)
                    }
                }
            }
            if (ability == "ender") {
                atc.teleport(vic.location)
            } else if ((ability == "poisonous") && (isLegitVictim(atc, playerIsVictom, ability))) {
                (vic as LivingEntity).addPotionEffect(PotionEffect(PotionEffectType.POISON, 200, 1))
            } else if ((ability == "morph") && (isLegitVictim(atc, playerIsVictom, ability))) {
                try {
                    var newEnt: Entity?
                    val mc = Random().nextInt(25) + 1
                    if (mc != 20) {
                        return
                    }
                    val l = atc.location.clone()
                    val h = (atc as org.bukkit.entity.Damageable).health
                    val aList =
                        infernalMobService.infernalList[infernalMobService.idSearch(id)]!!.abilityList
                    //Remove old
                    val dis = 46.0
                    for (e in atc.getNearbyEntities(dis, dis, dis)) if (e is Player) {
                        gui.fixBar(
                            e
                        )
                    }
                    atc.teleport(Location(atc.getWorld(), l.x, 0.0, l.z))
                    atc.remove()
                    plugin.logger.log(Level.INFO, "Entity remove due to Morph")
                    val mList = configService.mainConfig.enabledMobs
                    val index = Random().nextInt(mList.size)
                    val mobName = mList[index]

                    newEnt = null
                    var arrayOfEntityType: Array<EntityType>
                    val j = (EntityType.values().also { arrayOfEntityType = it }).size
                    for (i in 0 until j) {
                        val e = arrayOfEntityType[i]
                        try {
                            if ((e.getName() != null) && (e.getName().equals(mobName, ignoreCase = true))) {
                                newEnt = vic.world.spawnEntity(l, e)
                            }
                        } catch (ignored: Exception) {
                        }
                    }
                    if (newEnt == null) {
                        println("Infernal Mobs can't find mob type: $mobName!")
                        return
                    }
                    val newMob = if (aList!!.contains("oneup")) {
                        InfernalMob(newEnt, newEnt.uniqueId, true, aList, 2, infernalMobService.effect)
                    } else {
                        InfernalMob(newEnt, newEnt.uniqueId, true, aList, 1, infernalMobService.effect)
                    }
                    if (aList.contains("flying")) {
                        EntityUtils.makeFly(newEnt)
                    }
                    infernalMobService.infernalList[infernalMobService.idSearch(id)] = newMob
                    gui!!.setName(newEnt)

                    infernalMobService.giveMobGear(newEnt, true)

                    infernalMobService.addHealth(newEnt, aList)
                    if (h >= (newEnt as LivingEntity).getAttribute(Attribute.GENERIC_MAX_HEALTH)!!
                            .baseValue
                    ) {
                        return
                    }
                    (newEnt as org.bukkit.entity.Damageable).health = h
                } catch (ex: Exception) {
                    print("Morph Error: ")
                    ex.printStackTrace()
                }
            }
            if ((ability == "molten") && (isLegitVictim(atc, playerIsVictom, ability))) {
                vic.fireTicks = configService.mainConfig.moltenBurnLength * 20
            } else if ((ability == "blinding") && (isLegitVictim(atc, playerIsVictom, ability))) {
                (vic as LivingEntity).addPotionEffect(PotionEffect(PotionEffectType.BLINDNESS, 60, 1))
            } else if ((ability == "confusing") && (isLegitVictim(atc, playerIsVictom, ability))) {
                (vic as LivingEntity).addPotionEffect(PotionEffect(PotionEffectType.CONFUSION, 80, 2))
            } else if ((ability == "withering") && (isLegitVictim(atc, playerIsVictom, ability))) {
                (vic as LivingEntity).addPotionEffect(PotionEffect(PotionEffectType.WITHER, 180, 1))
            } else if ((ability == "thief") && (isLegitVictim(atc, playerIsVictom, ability))) {
                if ((vic is Player)) {
                    if ((vic.inventory.itemInMainHand.type != Material.AIR) && ((randomNum <= 1) || (randomNum == 1))) {
                        vic.getWorld().dropItemNaturally(atc.location, vic.inventory.itemInMainHand)
                        val slot = vic.inventory.heldItemSlot
                        vic.inventory.setItem(slot, null)
                    }
                } else if (vic is Zombie || vic is Skeleton) {
                    val eq = (vic as LivingEntity).equipment!!
                    vic.getWorld().dropItemNaturally(atc.location, eq.itemInMainHand)
                    eq.setItemInMainHand(null)
                }
            } else if ((ability == "quicksand") && (isLegitVictim(atc, playerIsVictom, ability))) {
                (vic as LivingEntity).addPotionEffect(PotionEffect(PotionEffectType.SLOW, 180, 1))
            } else if ((ability == "bullwark") && (isLegitVictim(atc, playerIsVictom, ability))) {
                (atc as LivingEntity).addPotionEffect(PotionEffect(PotionEffectType.DAMAGE_RESISTANCE, 500, 2))
            } else if ((ability == "rust") && (isLegitVictim(atc, playerIsVictom, ability))) {
                val damItem = (vic as Player).inventory.itemInMainHand
                if (((randomNum <= 3) || (randomNum == 1)) && (damItem.maxStackSize == 1)) {
                    val cDur = (damItem.itemMeta as Damageable).damage
                    (damItem.itemMeta as Damageable).damage = cDur + 20
                }
            } else if ((ability == "sapper") && (isLegitVictim(atc, playerIsVictom, ability))) {
                (vic as LivingEntity).addPotionEffect(PotionEffect(PotionEffectType.HUNGER, 500, 1), true)
            } else if ((ability != "oneup") || (!isLegitVictim(atc, playerIsVictom, ability))) {
                val needAir2: Location
                if ((ability == "ender") && (isLegitVictim(atc, playerIsVictom, ability))) {
                    val targetLocation = vic.location
                    if (randomNum >= 8) {
                        val rand2 = Random()
                        val min2 = 1
                        val max2 = 4
                        val randomNum2 = rand2.nextInt(max2 - min2 + 1) + min2
                        if (randomNum2 == 1) {
                            targetLocation.z = targetLocation.z + 6.0
                        } else if (randomNum2 == 2) {
                            targetLocation.z = targetLocation.z - 5.0
                        } else if (randomNum2 == 3) {
                            targetLocation.x = targetLocation.x + 8.0
                        } else if (randomNum2 == 4) {
                            targetLocation.x = targetLocation.x - 10.0
                        }
                        needAir2 = targetLocation
                        needAir2.y = needAir2.y + 1.0
                        targetLocation.y = targetLocation.y + 2.0
                        if (((targetLocation.block.type == Material.AIR) || (targetLocation.block.type == Material.TORCH)) &&
                            ((needAir2.block.type == Material.AIR) || (needAir2.block.type == Material.TORCH)) && ((targetLocation.block.type == Material.AIR) || (targetLocation.block.type == Material.TORCH))
                        ) {
                            atc.teleport(targetLocation)
                        }
                    }
                } else if ((ability == "lifesteal") && (isLegitVictim(atc, playerIsVictom, ability))) {
                    (atc as LivingEntity).addPotionEffect(PotionEffect(PotionEffectType.REGENERATION, 20, 1))
                } else if ((ability != "cloaked") || (!isLegitVictim(atc, playerIsVictom, ability))) {
                    if ((ability == "storm") && (isLegitVictim(atc, playerIsVictom, ability))) {
                        if (((randomNum <= 2) || (randomNum == 1)) && (!atc.isDead)) {
                            vic.world.strikeLightning(vic.location)
                        }
                    } else if ((ability != "sprint") || (!isLegitVictim(atc, playerIsVictom, ability))) {
                        if ((ability == "webber") && (isLegitVictim(atc, playerIsVictom, ability))) {
                            if ((randomNum >= 8) || (randomNum == 1)) {
                                val feet = vic.location
                                feet.block.type = Material.COBWEB
                                LocationUtils.setAir(feet, 60)

                                val rNum = Random().nextInt(max - min) + min
                                if ((rNum == 5) && ((atc.type == EntityType.SPIDER) || (atc.type == EntityType.CAVE_SPIDER))) {
                                    val l = atc.location
                                    val b = l.block
                                    val blocks = BlockUtils.getSphere(b)
                                    for (bl in blocks) {
                                        if (bl.type == Material.AIR) {
                                            bl.type = Material.COBWEB
                                            LocationUtils.setAir(bl.location, 30)
                                        }
                                    }
                                }
                            }
                        } else if ((ability == "vengeance") && (isLegitVictim(atc, playerIsVictom, ability))) {
                            if ((randomNum >= 5) || (randomNum == 1)) {
                                if ((vic is LivingEntity)) {
                                    vic.damage(Math.round(2.0 * configService.mainConfig.vengeanceDamage).toInt().toDouble())
                                }
                            }
                        } else if ((ability == "weakness") && (isLegitVictim(atc, playerIsVictom, ability))) {
                            (vic as LivingEntity).addPotionEffect(PotionEffect(PotionEffectType.WEAKNESS, 500, 1))
                        } else if ((ability == "berserk") && (isLegitVictim(atc, playerIsVictom, ability))) {
                            if ((randomNum >= 5) && (!atc.isDead)) {
                                val health = (atc as org.bukkit.entity.Damageable).health
                                atc.health = health - 1.0
                                if ((vic is LivingEntity)) {
                                    vic.damage(Math.round(2.0 * configService.mainConfig.berserkDamage).toInt().toDouble())
                                }
                            }
                        } else if ((ability == "potions") && (isLegitVictim(atc, playerIsVictom, ability))) {
                            val iStack = ItemStack(Material.POTION)
                            val potion = iStack.itemMeta as PotionMeta
                            when (randomNum) {
                                5 -> {
                                    potion.addCustomEffect(PotionEffect(PotionEffectType.HARM, 1, 2), true)
                                    potion.addCustomEffect(PotionEffect(PotionEffectType.HARM, 1, 1), true)
                                    potion.addCustomEffect(PotionEffect(PotionEffectType.WEAKNESS, (20 * 15), 2), true)
                                    potion.addCustomEffect(PotionEffect(PotionEffectType.POISON, (20 * 5), 2), true)
                                    potion.addCustomEffect(PotionEffect(PotionEffectType.SLOW, (20 * 10), 2), true)
                                }

                                6 -> {
                                    potion.addCustomEffect(PotionEffect(PotionEffectType.HARM, 1, 1), true)
                                    potion.addCustomEffect(PotionEffect(PotionEffectType.WEAKNESS, (20 * 15), 2), true)
                                    potion.addCustomEffect(PotionEffect(PotionEffectType.POISON, (20 * 5), 2), true)
                                    potion.addCustomEffect(PotionEffect(PotionEffectType.SLOW, (20 * 10), 2), true)
                                }

                                7 -> {
                                    potion.addCustomEffect(PotionEffect(PotionEffectType.WEAKNESS, (20 * 15), 2), true)
                                    potion.addCustomEffect(PotionEffect(PotionEffectType.POISON, (20 * 5), 2), true)
                                    potion.addCustomEffect(PotionEffect(PotionEffectType.SLOW, (20 * 10), 2), true)
                                }

                                8 -> {
                                    potion.addCustomEffect(PotionEffect(PotionEffectType.POISON, (20 * 5), 2), true)
                                    potion.addCustomEffect(PotionEffect(PotionEffectType.SLOW, (20 * 10), 2), true)
                                }

                                9 -> potion.addCustomEffect(PotionEffect(PotionEffectType.SLOW, (20 * 10), 2), true)
                            }
                            iStack.setItemMeta(potion)
                            val sploc = atc.location
                            sploc.y = sploc.y + 3.0
                            val thrownPotion = vic.world.spawnEntity(sploc, EntityType.SPLASH_POTION) as ThrownPotion
                            thrownPotion.item = iStack
                            val direction = atc.location.direction
                            direction.normalize()
                            direction.add(Vector(0.0, 0.2, 0.0))

                            var dist = atc.location.distance(vic.location)

                            dist /= 15.0
                            direction.multiply(dist)
                            thrownPotion.velocity = direction
                            //                }
                        } else if ((ability == "mama") && (isLegitVictim(atc, playerIsVictom, ability))) {
                            if (randomNum == 1) {
                                val amount = configService.mainConfig.mamaSpawnAmount
                                if (atc.type == EntityType.MUSHROOM_COW) {
                                    for (i in 0 until amount) {
                                        val minion =
                                            atc.world.spawnEntity(atc.location, EntityType.MUSHROOM_COW) as MushroomCow
                                        minion.setBaby()
                                    }
                                } else if (atc.type == EntityType.COW) {
                                    for (i in 0 until amount) {
                                        val minion = atc.world.spawnEntity(atc.location, EntityType.COW) as Cow
                                        minion.setBaby()
                                    }
                                } else if (atc.type == EntityType.SHEEP) {
                                    for (i in 0 until amount) {
                                        val minion = atc.world.spawnEntity(atc.location, EntityType.SHEEP) as Sheep
                                        minion.setBaby()
                                    }
                                } else if (atc.type == EntityType.PIG) {
                                    for (i in 0 until amount) {
                                        val minion = atc.world.spawnEntity(atc.location, EntityType.PIG) as Pig
                                        minion.setBaby()
                                    }
                                } else if (atc.type == EntityType.CHICKEN) {
                                    for (i in 0 until amount) {
                                        val minion = atc.world.spawnEntity(atc.location, EntityType.CHICKEN) as Chicken
                                        minion.setBaby()
                                    }
                                } else if (atc.type == EntityType.WOLF) {
                                    for (i in 0 until amount) {
                                        val minion = atc.world.spawnEntity(atc.location, EntityType.WOLF) as Wolf
                                        minion.setBaby()
                                    }
                                } else if (atc.type == EntityType.ZOMBIE) {
                                    for (i in 0 until amount) {
                                        val minion = atc.world.spawnEntity(atc.location, EntityType.ZOMBIE) as Zombie
                                        minion.setBaby()
                                    }
                                } else if (atc.type == EntityType.PIGLIN) {
                                    for (i in 0 until amount) {
                                        val minion = atc.world.spawnEntity(atc.location, EntityType.PIGLIN) as PigZombie
                                        minion.setBaby()
                                    }
                                } else if (atc.type == EntityType.OCELOT) {
                                    for (i in 0 until amount) {
                                        val minion = atc.world.spawnEntity(atc.location, EntityType.OCELOT) as Ocelot
                                        minion.setBaby()
                                    }
                                } else if (atc.type == EntityType.HORSE) {
                                    for (i in 0 until amount) {
                                        val minion = atc.world.spawnEntity(atc.location, EntityType.HORSE) as Horse
                                        minion.setBaby()
                                    }
                                } else if (atc.type == EntityType.VILLAGER) {
                                    for (i in 0 until amount) {
                                        val minion =
                                            atc.world.spawnEntity(atc.location, EntityType.VILLAGER) as Villager
                                        minion.setBaby()
                                    }
                                } else {
                                    for (i in 0 until amount) {
                                        atc.world.spawnEntity(atc.location, atc.type)
                                    }
                                }
                            }
                        } else if ((ability == "archer") && (isLegitVictim(atc, playerIsVictom, ability))) {
                            if ((randomNum > 7) || (randomNum == 1)) {
                                val arrowList = mutableListOf<Arrow>()
                                val loc1 = vic.location
                                val loc2 = atc.location
                                if (!EntityUtils.isSmall(atc)) {
                                    loc2.y += 1.0
                                }
                                val a = (atc as LivingEntity).launchProjectile(
                                    Arrow::class.java
                                )
                                val arrowSpeed = 1
                                loc2.y = (loc2.blockY + 2).toDouble()
                                loc2.x = loc2.blockX + 0.5
                                loc2.z = loc2.blockZ + 0.5
                                val a2 = a.world.spawnArrow(
                                    loc2,
                                    Vector(loc1.x - loc2.x, loc1.y - loc2.y, loc1.z - loc2.z),
                                    arrowSpeed.toFloat(),
                                    12.0f
                                )
                                a2.shooter = atc
                                loc2.y = (loc2.blockY + 2).toDouble()
                                loc2.x = (loc2.blockX - 1).toDouble()
                                loc2.z = (loc2.blockZ - 1).toDouble()
                                val a3 = a.world.spawnArrow(
                                    loc2,
                                    Vector(loc1.x - loc2.x, loc1.y - loc2.y, loc1.z - loc2.z),
                                    arrowSpeed.toFloat(),
                                    12.0f
                                )
                                a3.shooter = atc
                                arrowList.add(a)
                                arrowList.add(a2)
                                arrowList.add(a3)
                                for (ar in arrowList) {
                                    var minAngle = 6.283185307179586
                                    var minEntity: Entity? = null
                                    for (entity in atc.getNearbyEntities(64.0, 64.0, 64.0)) {
                                        if ((atc.hasLineOfSight(entity)) && ((entity is LivingEntity)) && (!entity.isDead())) {
                                            val toTarget = entity.getLocation().toVector().clone()
                                                .subtract(atc.getLocation().toVector())
                                            val angle = ar.velocity.angle(toTarget).toDouble()
                                            if (angle < minAngle) {
                                                minAngle = angle
                                                minEntity = entity
                                            }
                                        }
                                    }
                                    if (minEntity != null) {
                                        ArrowHomingTask(ar, minEntity as LivingEntity, plugin)
                                    }
                                }
                            }
                        } else if ((ability == "firework") && (isLegitVictim(atc, playerIsVictom, ability))) {
                            val red = configService.mainConfig.fireworkColour.red
                            val green = configService.mainConfig.fireworkColour.green
                            val blue = configService.mainConfig.fireworkColour.blue
                            val tmpCol = ItemStack(Material.LEATHER_HELMET, 1)
                            val tmpCol2 = tmpCol.itemMeta as LeatherArmorMeta
                            tmpCol2.setColor(Color.fromRGB(red, green, blue))

                            val col = tmpCol2.color
                            EntityUtils.launchFirework(atc.location, col, 1)
                        }
                    }
                }
            }
        } catch (ignored: Exception) {
        }
    }

    private fun isLegitVictim(e: Entity, playerIsVictom: Boolean, ability: String): Boolean {
        if ((e is Player)) {
            return true
        }
        if (configService.mainConfig.effectAllPlayerAttacks) {
            return true
        }
        val attackAbilityList = mutableListOf<String>()
        attackAbilityList.add("poisonous")
        attackAbilityList.add("blinding")
        attackAbilityList.add("withering")
        attackAbilityList.add("thief")
        attackAbilityList.add("sapper")
        attackAbilityList.add("lifesteal")
        attackAbilityList.add("storm")
        attackAbilityList.add("webber")
        attackAbilityList.add("weakness")
        attackAbilityList.add("berserk")
        attackAbilityList.add("potions")
        attackAbilityList.add("archer")
        attackAbilityList.add("confusing")
        if ((playerIsVictom) && (attackAbilityList.contains(ability))) {
            return true
        }
        val defendAbilityList = mutableListOf<String>()
        defendAbilityList.add("thief")
        defendAbilityList.add("storm")
        defendAbilityList.add("webber")
        defendAbilityList.add("weakness")
        defendAbilityList.add("potions")
        defendAbilityList.add("archer")
        defendAbilityList.add("quicksand")
        defendAbilityList.add("bullwark")
        defendAbilityList.add("rust")
        defendAbilityList.add("ender")
        defendAbilityList.add("vengeance")
        defendAbilityList.add("mama")
        defendAbilityList.add("firework")
        defendAbilityList.add("morph")
        return (!playerIsVictom) && (defendAbilityList.contains(ability))
    }
}