package io.github.thehrz.infernalmobs.listeners

import io.github.thehrz.infernalmobs.InfernalMobs
import io.github.thehrz.infernalmobs.events.InfernalSpawnEvent
import io.github.thehrz.infernalmobs.gui.GUI
import io.github.thehrz.infernalmobs.handler.EffectHandle
import io.github.thehrz.infernalmobs.handler.InfernalMobHandler
import io.github.thehrz.infernalmobs.models.InfernalMob
import io.github.thehrz.infernalmobs.services.ConfigService
import io.github.thehrz.infernalmobs.services.InfernalMobService
import io.github.thehrz.infernalmobs.utils.BlockUtils
import io.github.thehrz.infernalmobs.utils.EntityUtils
import io.github.thehrz.infernalmobs.utils.LocationUtils
import io.github.thehrz.infernalmobs.utils.MathUtils
import org.bukkit.Bukkit
import org.bukkit.ChatColor
import org.bukkit.Material
import org.bukkit.entity.Ageable
import org.bukkit.entity.Entity
import org.bukkit.entity.EntityType
import org.bukkit.entity.Player
import org.bukkit.event.EventHandler
import org.bukkit.event.EventPriority
import org.bukkit.event.Listener
import org.bukkit.event.entity.CreatureSpawnEvent
import org.koin.core.annotation.Single
import java.util.*
import java.util.logging.Level

@Single
class CreatureSpawnEventListener(
    val plugin: InfernalMobs,
    private val configService: ConfigService,
    private val infernalMobService: InfernalMobService,
    val infernalMobHandler: InfernalMobHandler,
    val effectHandle: EffectHandle,
    val gui: GUI
) : Listener {
    private val spawnerMap = HashMap<String, Long?>()

    @EventHandler(priority = EventPriority.HIGH)
    fun onMobSpawn(event: CreatureSpawnEvent) {
        val world = event.entity.world
        if ((!event.entity.hasMetadata("NPC")) && (!event.entity.hasMetadata("shopkeeper")) && event.entity.customName == null) {
            if (event.entity.type == EntityType.ENDER_DRAGON) plugin.logger
                .log(Level.INFO, "Detected Entity Spawn: Ender Dragon")
            if (event.spawnReason == CreatureSpawnEvent.SpawnReason.SPAWNER) {
                val spawner = BlockUtils.blockNear(event.entity.location, Material.SPAWNER, 10)
                if (spawner != null) {
                    val name: String = LocationUtils.getLocationName(spawner.location)
                    if (configService.mobSaveConfig.getString("infernalSpanwers.$name") != null) {
                        if (spawnerMap[name] == null) {
                            makeInfernal(event.entity, true)
                            spawnerMap[name] =
                                plugin.serverTime
                        } else {
                            val startTime = spawnerMap[name]!!
                            val endTime: Long = plugin.serverTime
                            val timePassed = endTime - startTime
                            val delay: Int = configService.mobSaveConfig.getInt(
                                "infernalSpanwers.$name"
                            )
                            if (timePassed >= delay) {
                                makeInfernal(event.entity, true)
                                spawnerMap[name] =
                                    plugin.serverTime
                            } else {
                                event.isCancelled = true
                            }
                        }
                    } /*else if (!plugin.getConfig().getBoolean("enableFarmingDrops")) {
						return;
					}*/
                }
            }
            //System.out.println("InfernalMob Spawn 2");
            if ((event.entity.hasMetadata("NPC")) || (event.entity.hasMetadata("shopkeeper"))) {
                return
            }
            val entName = event.entity.type.name
            if (((configService.mainConfig.mobWorlds
                    .contains(world.name)) || (configService.mainConfig.mobWorlds
                    .contains("<all>"))) &&
                (configService.mainConfig.enabledMobs.contains(entName)) &&
                (configService.mainConfig.naturalSpawnHeight < event.entity.location.y) &&
                (configService.mainConfig.enabledSpawnReasons
                    .contains(event.spawnReason.toString()))
            ) {
                makeInfernal(event.entity, false)
            }
        }
    }

    private fun makeInfernal(e: Entity, fixed: Boolean) {
        val entName = e.type.name
        if ((!e.hasMetadata("NPC")) && (!e.hasMetadata("shopkeeper"))) {
            if (!fixed) {
                val babyList = configService.mainConfig.disabledBabyMobs
                if (e is Ageable) {
                    val baby = !e.isAdult
                    if (baby && babyList.contains(entName)) {
                        return
                    }
                }
            }
            val id = e.uniqueId
            val chance = configService.mainConfig.chance
            Bukkit.getServer().scheduler.scheduleSyncDelayedTask(plugin, {
                val entName1 = e.type.name
                if ((!e.isDead) && (e.isValid) && (((configService.mainConfig.enabledMobs
                        .contains(entName1))) || ((fixed) &&
                            (infernalMobService.idSearch(id) == -1)))
                ) {
                    //Default
                    val min = 1
                    var max = chance
                    //Pe InfernalMob
                    val mc = plugin.config.getInt("mobChances.$entName1")
                    if (mc > 0) max = mc
                    if (fixed) max = 1
                    //int randomNum = new Random().nextInt(max - min) + min;
                    val randomNum = MathUtils.rand(min, max)
                    if (randomNum == 1) {
                        val aList = infernalMobHandler.getAbilitiesAmount(e)
                        //todo: levelChance
//                        if (configService.getString("levelChance." + aList.size) != null) {
//                            val sc =
//                                config.getInt("levelChance." + aList.size)
//                            val randomNum2 = Random().nextInt(sc - min) + min
//                            if (randomNum2 != 1) {
//                                return@scheduleSyncDelayedTask
//                            }
//                        }
                        val newMob = if (aList.contains("oneup")) {
                            InfernalMob(e, id, true, aList, 2, infernalMobService.effect)
                        } else {
                            InfernalMob(e, id, true, aList, 1, infernalMobService.effect)
                        }

                        //fire event
                        val infernalEvent = InfernalSpawnEvent(e, newMob)
                        Bukkit.getPluginManager().callEvent(infernalEvent)
                        if (infernalEvent.isCancelled) {
                            return@scheduleSyncDelayedTask
                        }

                        if (aList.contains("flying")) {
                            EntityUtils.makeFly(e)
                        }
                        infernalMobService.infernalList.add(newMob)
                        gui.setName(e)
                        infernalMobService.giveMobGear(e, true)
                        infernalMobService.addHealth(e, aList)
                        if (configService.mainConfig.enableSpawnMessages) {
                            val spawnMessageList = configService.messagesConfig.spawnMessages
                            val randomGenerator = Random()
                            val index = randomGenerator.nextInt(spawnMessageList.size)
                            var spawnMessage = spawnMessageList[index]

                            spawnMessage = ChatColor.translateAlternateColorCodes('&', spawnMessage)
                            spawnMessage = if (e.customName != null) {
                                spawnMessage.replace("mob", e.customName!!)
                            } else {
                                spawnMessage.replace("mob", e.type.toString().lowercase(Locale.getDefault()))
                            }
                            val r = configService.mainConfig.spawnMessageRadius
                            if (r == -1) {
                                for (p in e.world.players) {
                                    p.sendMessage(spawnMessage)
                                }
                            } else if (r == -2) {
                                Bukkit.broadcastMessage(spawnMessage)
                            } else {
                                for (e1 in e.getNearbyEntities(
                                    r.toDouble(),
                                    r.toDouble(),
                                    r.toDouble()
                                )) {
                                    if ((e1 is Player)) {
                                        e1.sendMessage(spawnMessage)
                                    }
                                }
                            }
                        }
                    }
                }
            }, 10L)
        }
    }


}