package io.github.thehrz.infernalmobs.listeners

import io.github.thehrz.infernalmobs.InfernalMobs
import io.github.thehrz.infernalmobs.gui.GUI
import io.github.thehrz.infernalmobs.handler.GhostHandler
import io.github.thehrz.infernalmobs.handler.InfernalMobHandler
import io.github.thehrz.infernalmobs.handler.LootHandler
import io.github.thehrz.infernalmobs.services.ConfigService
import io.github.thehrz.infernalmobs.services.InfernalMobService
import org.bukkit.*
import org.bukkit.entity.EntityType
import org.bukkit.entity.Item
import org.bukkit.entity.Player
import org.bukkit.entity.TNTPrimed
import org.bukkit.event.EventHandler
import org.bukkit.event.EventPriority
import org.bukkit.event.Listener
import org.bukkit.event.entity.EntityDeathEvent
import org.koin.core.annotation.Single
import java.util.*

@Single
class EntityDeathListener(
    val plugin: InfernalMobs,
    val configService: ConfigService,
    val infernalMobService: InfernalMobService,
    val ghostHandler: GhostHandler,
    val lootHandler: LootHandler,
    val infernalMobHandler: InfernalMobHandler,
    val gui: GUI
) : Listener {
    private val dropedLootList = mutableListOf<UUID>()

    @EventHandler(priority = EventPriority.HIGH)
    fun onEntityDeath(event: EntityDeathEvent) {
        val id = event.entity.uniqueId
        val mobIndex = infernalMobService.idSearch(id)
        if (mobIndex != -1) {
            val aList = infernalMobService.findMobAbilities(id)

            //ArrayList<String> aList;
            if (aList.contains("explode")) {
                val tnt = event.entity.world.spawnEntity(event.entity.location, EntityType.PRIMED_TNT) as TNTPrimed
                tnt.fuseTicks = 1
            }
            var isGhost = false
            if ((event.entity.equipment?.helmet?.itemMeta?.displayName ?: "") == "§fGhost Head") {
                isGhost = true
            }

            if (aList.contains("ghost")) {
                ghostHandler.spawnGhost(event.entity.location)
            }
            val dropSpot: Location?
            if (aList.contains("molten")) {
                val lavaSpot = event.entity.location
                dropSpot = lavaSpot
                dropSpot.x -= 2.0
            } else {
                dropSpot = event.entity.location
            }
            if ((configService.mainConfig.enableDeathMessages) && ((event.entity.killer is Player)) && (!isGhost)
            ) {
                val player = event.entity.killer
                if (configService.messagesConfig.deathMessages.isNotEmpty()) {
                    val deathMessagesList = configService.messagesConfig.deathMessages
                    val randomGenerator = Random()
                    val index = randomGenerator.nextInt(deathMessagesList.size)
                    var deathMessage = deathMessagesList[index]
                    val tittle = gui.getMobNameTag(event.entity)
                    deathMessage = ChatColor.translateAlternateColorCodes('&', deathMessage)
                    deathMessage = deathMessage.replace("player", player!!.name)
                    deathMessage =
                        if (player.itemInHand.type != Material.AIR) {
                            deathMessage.replace("weapon", player.itemInHand.itemMeta.displayName)
                        } else {
                            deathMessage.replace("weapon", "fist")
                        }
                    deathMessage = if (event.entity.customName != null) {
                        deathMessage.replace("mob", event.entity.customName!!)
                    } else {
                        deathMessage.replace("mob", tittle)
                    }
                    Bukkit.broadcastMessage(deathMessage)
                } else {
                    println("No valid death messages found!")
                }
            }
            if ((configService.mainConfig.enableDrops)
                &&
                ((configService.mainConfig.enableFarmingDrops)
                        || (event.entity.killer != null))
                &&
                ((configService.mainConfig.enableFarmingDrops)
                        || ((event.entity.killer is Player)))
            ) {
                val player = event.entity.killer!!

                if ((player.gameMode == GameMode.CREATIVE) && configService.mainConfig.noCreativeDrops) {
                    return
                }
                val drop = lootHandler.getRandomLoot(player, event.entity.type.getName(), aList.size)
                if (drop != null) {
                    val min = 1
                    val max: Int = configService.mainConfig.dropChance
                    val randomNum = Random().nextInt(max - min + 1) + min
                    if (randomNum == 1) {
                        val dropedItem = event.entity.world.dropItemNaturally(dropSpot, drop)
                        keepAlive(dropedItem)
                    }
                    val xpm: Int = configService.mainConfig.xpMultiplier
                    val xp = event.droppedExp * xpm
                    event.droppedExp = xp
                }
            }
            try {
                infernalMobHandler.removeMob(mobIndex)
            } catch (e: Exception) {
                println("Error: $e")
            }
        }
        return

    }

    private fun keepAlive(item: Item) {
        val id = item.uniqueId
        dropedLootList.add(id)
        Bukkit.getServer().scheduler.scheduleSyncDelayedTask(
            plugin,
            { dropedLootList.remove(id) }, 300L
        )
    }
}