package io.github.thehrz.infernalmobs.services

import de.exlll.configlib.ConfigLib
import de.exlll.configlib.NameFormatters
import de.exlll.configlib.YamlConfigurationProperties
import de.exlll.configlib.YamlConfigurations
import io.github.thehrz.infernalmobs.InfernalMobs
import io.github.thehrz.infernalmobs.config.MainConfig
import io.github.thehrz.infernalmobs.config.MessagesConfig
import org.bukkit.Material
import org.bukkit.block.Banner
import org.bukkit.configuration.file.FileConfiguration
import org.bukkit.configuration.file.YamlConfiguration
import org.bukkit.enchantments.Enchantment
import org.bukkit.inventory.ItemStack
import org.bukkit.inventory.meta.*
import org.koin.core.annotation.Single
import java.io.File
import java.io.IOException

@Single
class ConfigService(private val plugin: InfernalMobs) {
    private val properties: YamlConfigurationProperties = ConfigLib.BUKKIT_DEFAULT_PROPERTIES.toBuilder()
        .setNameFormatter(NameFormatters.IDENTITY)
        .build()

    private val mobSaveYML = File(plugin.dataFolder, "save.yml")
    private val lootYML = File(plugin.dataFolder, "loot.yml")

    lateinit var mainConfig: MainConfig
    lateinit var messagesConfig: MessagesConfig
    lateinit var mobSaveConfig: YamlConfiguration
    lateinit var lootConfig: YamlConfiguration

    fun loadConfigs() {
        val configFile = File(plugin.dataFolder, "config.yml").toPath()

        mainConfig = YamlConfigurations.update(
            configFile,
            MainConfig::class.java,
            properties
        )

        val messagesFile = File(plugin.dataFolder, "messages.yml").toPath()
        messagesConfig = YamlConfigurations.update(
            messagesFile,
            MessagesConfig::class.java,
            properties
        )

        if (!mobSaveYML.exists()) {
            plugin.saveResource("save.yml", false);
        }
        mobSaveConfig = YamlConfiguration.loadConfiguration(mobSaveYML)

        if (!lootYML.exists()) {
            plugin.saveResource("loot.yml", false);
        }
        lootConfig = YamlConfiguration.loadConfiguration(lootYML)
    }

    fun saveLoot() {
        lootConfig.save(lootYML)
    }

    fun saveSave() {
        mobSaveConfig.save(mobSaveYML)
    }

    fun setItem(s: ItemStack?, path: String, fc: FileConfiguration) {
        if (s != null) {
            fc["$path.item"] = s.type.toString()
            fc["$path.amount"] = s.amount
            fc["$path.durability"] = (s as Damageable).damage
            if (s.itemMeta != null) {
                fc["$path.name"] = s.itemMeta.displayName
                if (s.itemMeta.lore != null) {
                    for (l in s.itemMeta.lore!!.indices) {
                        if (s.itemMeta.lore!![l] != null) {
                            fc["$path.lore$l"] = s.itemMeta.lore!![l]
                        }
                    }
                }
            }
            var e: Enchantment
            for ((key, level) in s.enchantments) {
                e = key
                for (ei in 0..12) {
                    if (fc.getString("$path.enchantments.$ei") == null) {
                        fc["$path.enchantments.$ei.enchantment"] = e.key
                        fc["$path.enchantments.$ei.level"] = level
                        break
                    }
                }
            }
            if (s.type == Material.ENCHANTED_BOOK) {
                val em = s.itemMeta as EnchantmentStorageMeta
                for (hm in em.storedEnchants.entries) {
                    e = (hm as Map.Entry<*, *>).key as Enchantment
                    val level = (hm as Map.Entry<*, *>).value as Int
                    for (ei in 0..12) {
                        if (fc.getString("$path.enchantments.$ei") == null) {
                            fc["$path.enchantments.$ei.enchantment"] = e.toString()
                            fc["$path.enchantments.$ei.level"] = level
                            break
                        }
                    }
                }
            }
            if ((s.type == Material.WRITTEN_BOOK) || (s.type == Material.WRITABLE_BOOK)) {
                val meta = s.itemMeta as BookMeta
                if (meta.author != null) {
                    fc["$path.author"] = meta.author
                }
                if (meta.title != null) {
                    fc["$path.title"] = meta.title
                }
                var i = 0
                for (p in meta.pages) {
                    fc["$path.pages.$i"] = p
                    i++
                }
            }
            //Banner
            if (s.type.toString().contains("BANNER")) {
                val b = s.itemMeta as BannerMeta
                if (b != null) {
                    val patList: List<*> = b.patterns
                    if (!patList.isEmpty()) fc["$path.patterns"] = patList
                }
            }
            //Shield
            if (s.type == Material.SHIELD) {
                val im = s.itemMeta
                val bmeta = im as BlockStateMeta
                val b = bmeta.blockState as Banner

                fc["$path.colour"] = b.baseColor.toString()
                val patList: List<*> = b.patterns
                if (!patList.isEmpty()) fc["$path.patterns"] = patList
            }
            //Potions
            if (s.type == Material.POTION || s.type == Material.SPLASH_POTION || s.type == Material.LINGERING_POTION) {
                val pMeta = s.itemMeta as PotionMeta
                val pd = pMeta.basePotionData
                fc["$path.potion"] = pd.type.effectType!!.name
            }
            if ((s.type == Material.LEATHER_BOOTS) || (s.type == Material.LEATHER_CHESTPLATE) || (s.type == Material.LEATHER_HELMET) || (s.type == Material.LEATHER_LEGGINGS)) {
                val l = s.itemMeta as LeatherArmorMeta
                val c = l.color
                val color = c.red.toString() + "," + c.green + "," + c.blue
                fc["$path.colour"] = color
            }
            if (s.type == Material.PLAYER_HEAD) {
                val sm = s.itemMeta as SkullMeta
                fc["$path.owner"] = sm.owningPlayer!!.uniqueId.toString()
            }
            val flags = ArrayList<String>()
            for (f in s.itemMeta.itemFlags) if (f != null) flags.add(f.name)
            if (flags.isNotEmpty()) fc["$path.flags"] = flags
        } else {
            println("Item is null!")
        }
        try {
            lootConfig.save(this.lootYML)
        } catch (ignored: IOException) {
        }
//        plugin.saveConfig()
    }

    fun reloadLoot() {
        val configFile = File(plugin.dataFolder, "config.yml").toPath()

        mainConfig = YamlConfigurations.load(
            configFile,
            MainConfig::class.java,
            properties
        )

        val messagesFile = File(plugin.dataFolder, "messages.yml").toPath()
        messagesConfig = YamlConfigurations.load(
            messagesFile,
            MessagesConfig::class.java,
            properties
        )

        mobSaveConfig = YamlConfiguration.loadConfiguration(mobSaveYML)

        lootConfig = YamlConfiguration.loadConfiguration(lootYML)
    }
}