package io.github.thehrz.infernalmobs.services

import io.github.thehrz.infernalmobs.InfernalMobs
import io.github.thehrz.infernalmobs.models.InfernalMob
import io.github.thehrz.infernalmobs.config.MainConfig
import io.github.thehrz.infernalmobs.utils.EntityUtils
import io.github.thehrz.infernalmobs.utils.MathUtils
import org.bukkit.DyeColor
import org.bukkit.Material
import org.bukkit.World
import org.bukkit.attribute.Attribute
import org.bukkit.enchantments.Enchantment
import org.bukkit.entity.*
import org.bukkit.inventory.ItemStack
import org.bukkit.metadata.FixedMetadataValue
import org.koin.core.annotation.Single
import java.io.IOException
import java.util.*

@Single
class InfernalMobService(private val configService: ConfigService, val plugin: InfernalMobs) {
    var infernalList = mutableListOf<InfernalMob>()
    val mountList = mutableMapOf<Entity, Entity>()

    val effect: String
        get() {
            var effect = "mobSpawnerFire"
            try {
                //Get Enabled Particles
                val partTypes = configService.mainConfig.mobParticles
                //Get Random Particle
                effect = partTypes[Random().nextInt(partTypes.size)]
            } catch (e: Exception) {
                println("Error: $e")
            }
            return effect
        }

    fun idSearch(id: UUID): Int {
        var idMob: InfernalMob? = null
        for (mob in this.infernalList) {
            if (mob.id == id) {
                idMob = mob
            }
        }
        if (idMob != null) {
            return infernalList.indexOf(idMob)
        }
        return -1
    }

    fun giveMobsPowers(world: World) {
        for (ent in world.entities) {
            if (((ent is LivingEntity)) && (configService.mobSaveConfig.getString(ent.getUniqueId().toString()) != null)) {
                giveMobPowers(ent)
            }
        }
    }

    fun giveMobPowers(ent: Entity) {
        val id = ent.uniqueId
        if (idSearch(id) == -1) {
            val metadata = ent.getMetadata("infernalMetadata")
            if (metadata.isNotEmpty()) {
                val aList = metadata.first().asString().split(",").filter { it.isNotEmpty() }

                val newMob = if (aList.contains("oneup")) {
                    InfernalMob(ent, id, true, aList, 2, effect)
                } else {
                    InfernalMob(ent, id, true, aList, 1, effect)
                }
                if (aList.contains("flying")) {
                    EntityUtils.makeFly(ent)
                }
                infernalList.add(newMob)
            }
        }
    }

    fun findMobAbilities(id: UUID): List<String> {
        for (mob in this.infernalList) {
            if (mob.id == id) {
                return mob.abilityList
            }
        }
        return emptyList()
    }

    fun getNearbyBoss(p: Player): Entity? {
        val dis = 26.0
        for (m in infernalList) {
            if (m.entity.world == p.world) {
                val boss = m.entity
                if (p.location.distance(boss.location) < dis) {
//                    dis = p.location.distance(boss.location)
                    return boss
                }
            }
        }
        return null
    }

    fun reloadPowers() {
        val wList = mutableListOf<World>()
        for (p in plugin.server.onlinePlayers) {
            if (!wList.contains(p.world)) {
                wList.add(p.world)
            }
        }
        for (world in wList) {
            giveMobsPowers(world)
        }
    }

    fun addHealth(ent: Entity, powerList: List<String?>) {
        //double maxHealth = ((org.bukkit.entity.Damageable) ent).getHealth();
        val maxHealth = (ent as LivingEntity).getAttribute(Attribute.GENERIC_MAX_HEALTH)!!
            .baseValue
        var setHealth: Float
        if (configService.mainConfig.healthByPower) {
            val mobIndex = idSearch(ent.getUniqueId())
            try {
                val m = infernalList[mobIndex]
                setHealth = (maxHealth * m.abilityList.size).toFloat()
            } catch (e: Exception) {
                setHealth = (maxHealth * 5.0).toFloat()
            }
        } else {
            if (configService.mainConfig.healthByDistance) {
                val l = ent.getWorld().spawnLocation
                var m = l.distance(ent.getLocation()).toInt() / configService.mainConfig.addDistance
                if (m < 1) {
                    m = 1
                }
                val add = configService.mainConfig.healthToAdd
                setHealth = (m * add).toFloat()
            } else {
                val healthMultiplier = configService.mainConfig.healthMultiplier
                setHealth = (maxHealth * healthMultiplier).toFloat()
            }
        }
        if (setHealth >= 1.0f) {
            try {
                ent.getAttribute(Attribute.GENERIC_MAX_HEALTH)!!.baseValue =
                    setHealth.toDouble()
                ent.health = setHealth.toDouble()
            } catch (e: Exception) {
                println("addHealth: $e")
            }
        }
        val list = getPowerString(ent, powerList)
        ent.setMetadata("infernalMetadata", FixedMetadataValue(plugin, list))
        try {
            configService.mobSaveConfig[ent.getUniqueId().toString()] = list
            configService.saveSave()
        } catch (ignored: IOException) {
        }
    }

    private fun getPowerString(ent: Entity, powerList: List<String?>): String {
        val list = StringBuilder()
        for (s in powerList) {
            if (powerList.indexOf(s) != powerList.size - 1) {
                list.append(s).append(",")
            } else {
                list.append(s)
            }
        }
        return list.toString()
    }

    fun giveMobGear(mob: Entity, naturalSpawn: Boolean) {
        val mobId = mob.uniqueId
        var mobAbilityList: List<String?>? = null
        var armoured = false
        if (idSearch(mobId) != -1) {
            mobAbilityList = findMobAbilities(mobId)
            if (mobAbilityList!!.contains("armoured")) {
                armoured = true
                (mob as LivingEntity).canPickupItems = false
            }
        }
        val helm = ItemStack(Material.DIAMOND_HELMET, 1)
        val chest = ItemStack(Material.DIAMOND_CHESTPLATE, 1)
        val pants = ItemStack(Material.DIAMOND_LEGGINGS, 1)
        val boots = ItemStack(Material.DIAMOND_BOOTS, 1)
        val sword = ItemStack(Material.DIAMOND_SWORD, 1)
        sword.addUnsafeEnchantment(Enchantment.DAMAGE_ALL, 4)
        val ee = (mob as LivingEntity).equipment
        if (mob.getType() == EntityType.WITHER_SKELETON) {
            if (armoured) {
                ee!!.helmetDropChance = 0.0f
                ee.chestplateDropChance = 0.0f
                ee.leggingsDropChance = 0.0f
                ee.bootsDropChance = 0.0f
                ee.itemInMainHandDropChance = 0.0f
                ee.helmet = helm
                ee.chestplate = chest
                ee.leggings = pants
                ee.boots = boots
                ee.setItemInMainHand(sword)
            }
        } else if (mob.getType() == EntityType.SKELETON) {
            val bow = ItemStack(Material.BOW, 1)
            ee!!.setItemInMainHand(bow)
            if (armoured) {
                ee.helmetDropChance = 0.0f
                ee.chestplateDropChance = 0.0f
                ee.helmet = helm
                ee.chestplate = chest
                if (!mobAbilityList!!.contains("cloaked")) {
                    ee.leggingsDropChance = 0.0f
                    ee.bootsDropChance = 0.0f
                    ee.leggings = pants
                    ee.boots = boots
                }
                ee.itemInMainHandDropChance = 0.0f
                ee.setItemInMainHand(sword)
            } else if (mobAbilityList!!.contains("cloaked")) {
                val skull = ItemStack(Material.GLASS_BOTTLE, 1)
                ee.helmet = skull
            }
        } else if (mob is Zombie) {
            if (armoured) {
                ee!!.helmetDropChance = 0.0f
                ee.chestplateDropChance = 0.0f
                ee.helmet = helm
                ee.chestplate = chest
                if (!mobAbilityList!!.contains("cloaked")) {
                    ee.leggings = pants
                    ee.boots = boots
                }
                ee.leggingsDropChance = 0.0f
                ee.bootsDropChance = 0.0f
                ee.itemInMainHandDropChance = 0.0f
                ee.setItemInMainHand(sword)
            } else if (mobAbilityList!!.contains("cloaked")) {
                val skull = ItemStack(Material.GLASS_BOTTLE)
                //skull.setDurability((short) 2);
                ee!!.helmet = skull
            }
        }
        if (((mobAbilityList!!.contains("mounted")) && (configService.mainConfig.enabledRiders
                .contains(mob.getType().name))) || ((!naturalSpawn) && (mobAbilityList.contains("mounted")))
        ) {
            val mounts = configService.mainConfig.enabledMounts

            val randomGenerator = Random()
            val index = randomGenerator.nextInt(mounts.size)
            var mount = mounts[index]
            //Type
            var type: String? = null
            if (mount.contains(":")) {
                val s = mount.split(":".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
                mount = s[0]
                type = s[1]
            }
            if (EntityType.fromName(mount) != null && (EntityType.fromName(mount) != EntityType.ENDER_DRAGON)) {
                val liveMount = mob.getWorld().spawnEntity(
                    mob.getLocation(),
                    EntityType.fromName(mount)!!
                )

                mountList[liveMount] = mob
                liveMount.addPassenger(mob)
                if (liveMount.type == EntityType.HORSE) {
                    val hm = liveMount as Horse
                    if (configService.mainConfig.horseMountsHaveSaddles) {
                        val saddle = ItemStack(Material.SADDLE)
                        hm.inventory.saddle = saddle
                    }
                    hm.isTamed = true
                    val randomNum3 = MathUtils.rand(1, 7)
                    if (randomNum3 == 1) {
                        hm.color = Horse.Color.BLACK
                    } else if (randomNum3 == 2) {
                        hm.color = Horse.Color.BROWN
                    } else if (randomNum3 == 3) {
                        hm.color = Horse.Color.CHESTNUT
                    } else if (randomNum3 == 4) {
                        hm.color = Horse.Color.CREAMY
                    } else if (randomNum3 == 5) {
                        hm.color = Horse.Color.DARK_BROWN
                    } else if (randomNum3 == 6) {
                        hm.color = Horse.Color.GRAY
                    } else {
                        hm.color = Horse.Color.WHITE
                    }
                    if ((armoured) && (configService.mainConfig.armouredMountsHaveArmour)) {
                        val armour = ItemStack(Material.DIAMOND_HORSE_ARMOR, 1)
                        hm.inventory.armor = armour
                    }
                } else if (liveMount.type == EntityType.SHEEP) {
                    val sh = liveMount as Sheep
                    if (type != null) {
                        sh.color = DyeColor.valueOf(type)
                    }
                }
            } else {
                println("Can't spawn mount!")
                println("$mount is not a valid Entity!")
            }
        }
    }

}