package io.github.thehrz.infernalmobs.utils

import io.github.thehrz.infernalmobs.InfernalMobs
import org.bukkit.Bukkit
import org.bukkit.Color
import org.bukkit.FireworkEffect
import org.bukkit.Location
import org.bukkit.entity.*
import org.bukkit.potion.PotionEffect
import org.bukkit.potion.PotionEffectType
import org.bukkit.util.Vector

object EntityUtils {
    fun getVector(entity: Entity): Vector {
        return if (entity is Player) entity.eyeLocation.toVector()
        else entity.location.toVector()
    }

    fun makeFly(ent: Entity) {
        val bat = ent.world.spawnEntity(ent.location, EntityType.BAT)
        bat.velocity = Vector(0, 1, 0)
        bat.addPassenger(ent)
        (bat as LivingEntity).addPotionEffect(PotionEffect(PotionEffectType.INVISIBILITY, 999999, 1))
    }

    fun isBaby(mob: Entity?): Boolean {
        if (mob is Ageable) {
            return !mob.isAdult
        }
        return false
    }

    fun isSmall(mob: Entity?): Boolean {
        return (isBaby(mob)) && (mob!!.type == EntityType.BAT) && (mob.type == EntityType.CAVE_SPIDER) && (mob.type == EntityType.CHICKEN) && (mob.type == EntityType.COW) && (mob.type == EntityType.MUSHROOM_COW) && (mob.type == EntityType.PIG) && (mob.type == EntityType.OCELOT) && (mob.type == EntityType.SHEEP) && (mob.type == EntityType.SILVERFISH) && (mob.type == EntityType.SPIDER) && (mob.type == EntityType.WOLF)
    }

    fun levitate(e: Entity, time: Int) {
        if ((e is LivingEntity)) {
            e.addPotionEffect(PotionEffect(PotionEffectType.LEVITATION, time * 20, 0))
        }
    }

    fun launchFirework(l: Location, c: Color, speed: Int) {
        val fw = l.world.spawn(l, Firework::class.java)
        val meta = fw.fireworkMeta
        meta.addEffect(FireworkEffect.builder().withColor(c).with(FireworkEffect.Type.BALL_LARGE).build())
        fw.fireworkMeta = meta
        fw.velocity = l.direction.multiply(speed)
        detonate(fw)
    }

    fun moveToward(e: Entity, to: Location, speed: Double) {
        if (e.isDead) {
            return
        }
        val direction = to.toVector().subtract(e.location.toVector()).normalize()
        e.velocity = direction.multiply(speed)
        Bukkit.getServer().scheduler.scheduleSyncDelayedTask(InfernalMobs.instance, {
            moveToward(e, to, speed)
        }, 1L)
    }

    private fun detonate(fw: Firework) {
        Bukkit.getServer().scheduler.scheduleSyncDelayedTask(InfernalMobs.instance, {
            fw.detonate()
        }, 2L)
    }
}